@@ -26,15 +26,15 @@
 #
 ##############################################################################
 import time
-import sql_db
+import pooler
 from report import report_sxw
 
 #
 # Use period and Journal for selection or resources
 #
 class account_analytic_journal(report_sxw.rml_parse):
-	def __init__(self, name, table, rml):
-		super(account_analytic_journal, self).__init__(name, table, rml)
+	def __init__(self, cr, uid, name, context):
+		super(account_analytic_journal, self).__init__(cr, uid, name, context)
 		self.localcontext.update( {
 			'time': time,
 			'lines': self._lines,
