@@ -42,12 +42,18 @@ class partner_balance(report_sxw.rml_parse):
 			'sum_scredit': self._sum_scredit,
 			'solde_debit': self._solde_balance_debit,
 			'solde_credit': self._solde_balance_credit,
+			'get_company': self._get_company,
+			'get_currency': self._get_currency,
 		})
 
 	def preprocess(self, objects, data, ids):
-		self.cr.execute('select distinct partner_id from account_move_line where partner_id is not null and date>=%s and date<=%s', (data['form']['date1'], data['form']['date2']))
+		self.cr.execute('select distinct line.partner_id \
+				from account_move_line AS line, account_account AS account \
+				where line.partner_id is not null and line.date>=%s and line.date<=%s \
+				and line.period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d) \
+				AND line.account_id = account.id AND account.company_id = %d AND account.active', (data['form']['date1'], data['form']['date2'], data['form']['fiscalyear'], data['form']['company_id']))
 		new_ids = [id for (id,) in self.cr.fetchall()]
-		self.cr.execute("SELECT a.id FROM account_account a LEFT JOIN account_account_type t ON (a.type=t.code) WHERE t.partner_account=TRUE")
+		self.cr.execute("SELECT a.id FROM account_account a LEFT JOIN account_account_type t ON (a.type=t.code) WHERE t.partner_account=TRUE AND a.company_id = %d AND a.active", (data['form']['company_id'],))
 		self.account_ids = ','.join([str(a) for (a,) in self.cr.fetchall()])
 		self.partner_ids = ','.join(map(str, new_ids))
 		objects = self.pool.get('res.partner').browse(self.cr, self.uid, new_ids)
@@ -58,18 +64,18 @@ class partner_balance(report_sxw.rml_parse):
 			return []
 
 #TODO: use an SQL CASE to compute sdebit and scredit
-#CHECKME: est-ce que les "enlitige", il fautdrait pas tester l etat de la ligne aussi?
 		self.cr.execute(
 			"SELECT p.ref, p.name, sum(debit) as debit, sum(credit) as credit, " \
-			"(SELECT sum(debit-credit) FROM account_move_line WHERE partner_id=p.id AND date>=%s AND date<=%s AND blocked=TRUE) as enlitige " \
+			"(SELECT sum(debit-credit) FROM account_move_line WHERE partner_id=p.id AND date>=%s AND date<=%s AND blocked=TRUE AND state <>'draft' AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d)) as enlitige " \
 			"FROM account_move_line l LEFT JOIN res_partner p ON (l.partner_id=p.id) " \
 			"WHERE partner_id IN (" + self.partner_ids + ") " \
 			"AND account_id IN (" + self.account_ids + ") " \
 			"AND l.date>=%s AND l.date<=%s " \
 			"AND l.state<>'draft' " \
+			"AND l.period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d) " \
 			"GROUP BY p.id, p.ref, p.name " \
 			"ORDER BY p.ref, p.name",
-			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['date1'], self.datas['form']['date2']))
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear'], self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		res = self.cr.dictfetchall()
 		for r in res:
 			r['sdebit'] = r['debit'] > r['credit'] and r['debit'] - r['credit']
@@ -86,8 +92,9 @@ class partner_balance(report_sxw.rml_parse):
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
 			"AND state<>'draft' " \
-			'AND date>=%s AND date<=%s', 
-			(self.datas['form']['date1'], self.datas['form']['date2']))
+			'AND date>=%s AND date<=%s ' \
+			"AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d)",
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit(self):
@@ -100,8 +107,9 @@ class partner_balance(report_sxw.rml_parse):
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
 			"AND state<>'draft' " \
-			'AND date>=%s AND date<=%s', 
-			(self.datas['form']['date1'], self.datas['form']['date2']))
+			'AND date>=%s AND date<=%s ' \
+			'AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d)',
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		return self.cr.fetchone()[0] or 0.0
 
 	def _sum_litige(self):
@@ -114,8 +122,9 @@ class partner_balance(report_sxw.rml_parse):
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
 			"AND state<>'draft' " \
-			'AND date>=%s AND date<=%s AND blocked=TRUE', 
-			(self.datas['form']['date1'], self.datas['form']['date2']))
+			'AND date>=%s AND date<=%s AND blocked=TRUE ' \
+			'AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d)',
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		return self.cr.fetchone()[0] or 0.0
 	
 	def _sum_sdebit(self):
@@ -131,10 +140,11 @@ class partner_balance(report_sxw.rml_parse):
 				'AND account_id IN (' + self.account_ids + ') ' \
 				'AND date>=%s AND date<=%s ' \
 				"AND state<>'draft' " \
+				"AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d) " \
 				'GROUP BY partner_id' \
 			') AS dc (debit,credit) ' \
 			'WHERE debit>credit', 
-			(self.datas['form']['date1'], self.datas['form']['date2']))
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_scredit(self):
@@ -150,10 +160,11 @@ class partner_balance(report_sxw.rml_parse):
 				'AND account_id IN (' + self.account_ids + ') ' \
 				'AND date>=%s AND date<=%s ' \
 				"AND state<>'draft' " \
+				"AND period_id in (SELECT id FROM account_period WHERE fiscalyear_id=%d) " \
 				'GROUP BY partner_id' \
 			') AS dc (debit,credit) ' \
 			'WHERE credit>debit', 
-			(self.datas['form']['date1'], self.datas['form']['date2']))
+			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['fiscalyear']))
 		return self.cr.fetchone()[0] or 0.0
 
 	def _solde_balance_debit(self):
@@ -163,6 +174,14 @@ class partner_balance(report_sxw.rml_parse):
 	def _solde_balance_credit(self):
 		debit, credit = self._sum_debit(), self._sum_credit()
 		return credit > debit and credit - debit
-	
-report_sxw.report_sxw('report.account.partner.balance', 'res.partner', 'addons/account/report/partner_balance.rml',parser=partner_balance)
+
+	def _get_company(self, form):
+		return pooler.get_pool(self.cr.dbname).get('res.company').browse(self.cr, self.uid, form['company_id']).name
+
+	def _get_currency(self, form):
+		return pooler.get_pool(self.cr.dbname).get('res.company').browse(self.cr, self.uid, form['company_id']).currency_id.name
+
+report_sxw.report_sxw('report.account.partner.balance', 'res.partner',
+	'addons/account/report/partner_balance.rml',parser=partner_balance,
+	header=False)
 
