@@ -25,24 +25,29 @@
 #
 ##############################################################################
 
-import time
 import wizard
+import pooler
 
 dates_form = '''<?xml version="1.0"?>
 <form string="Select period">
-	<field name="date1"/>
-	<field name="date2"/>
+	<field name="fiscalyear" colspan="4"/>
+	<field name="periods" colspan="4"/>
 </form>'''
 
 dates_fields = {
-	'date1': {'string':'Start of period', 'type':'date', 'required':True, 'default': lambda *a: time.strftime('%Y-01-01')},
-	'date2': {'string':'End of period', 'type':'date', 'required':True, 'default': lambda *a: time.strftime('%Y-%m-%d')},
+	'fiscalyear': {'string': 'Fiscal year', 'type': 'many2one', 'relation': 'account.fiscalyear', 'required': True},
+	'periods': {'string': 'Periods', 'type': 'many2many', 'relation': 'account.period', 'help': 'All periods if empty'}
 }
 
 class wizard_report(wizard.interface):
+	def _get_defaults(self, cr, uid, data, context):
+		fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
+		data['form']['fiscalyear'] = fiscalyear_obj.find(cr, uid)
+		return data['form']
+
 	states = {
 		'init': {
-			'actions': [], 
+			'actions': [_get_defaults],
 			'result': {'type':'form', 'arch':dates_form, 'fields':dates_fields, 'state':[('end','Cancel'),('report','Print')]}
 		},
 		'report': {
