@@ -42,6 +42,7 @@ create table ir_values
 
 CREATE TABLE ir_model (
   id serial,
+  perm_id int references perm on delete set null,
   model varchar(64) DEFAULT ''::varchar NOT NULL,
   name varchar(64),
   info text,
@@ -50,6 +51,7 @@ CREATE TABLE ir_model (
 
 CREATE TABLE ir_model_fields (
   id serial,
+  perm_id int references perm on delete set null,
   model varchar(64) DEFAULT ''::varchar NOT NULL,
   model_id int references ir_model,
   name varchar(64) DEFAULT ''::varchar NOT NULL,
@@ -90,18 +92,21 @@ CREATE TABLE ir_act_report_xml (
     report_name varchar(64) NOT NULL,
     report_xsl varchar(64),
     report_xml varchar(64),
-	auto boolean default true
+    auto boolean default true,
+    primary key(id)
 )
 INHERITS (ir_actions);
 
 create table ir_act_report_custom (
-    report_id int
+    report_id int,
 --  report_id int references ir_report_custom
+    primary key(id)
 )
 INHERITS (ir_actions);
 
 CREATE TABLE ir_act_group (
-    exec_type varchar(64) DEFAULT 'serial'::varchar NOT NULL
+    exec_type varchar(64) DEFAULT 'serial'::varchar NOT NULL,
+    primary key(id)
 )
 INHERITS (ir_actions);
 
@@ -112,12 +117,14 @@ CREATE TABLE ir_act_group_link (
 
 CREATE TABLE ir_act_execute (
     func_name varchar(64) NOT NULL,
-    func_arg varchar(64)
+    func_arg varchar(64),
+    primary key(id)
 )
 INHERITS (ir_actions);
 
 CREATE TABLE ir_act_wizard (
-    wiz_name varchar(64) NOT NULL
+    wiz_name varchar(64) NOT NULL,
+    primary key(id)
 )
 INHERITS (ir_actions);
 
@@ -130,8 +137,7 @@ CREATE TABLE ir_ui_view (
 	arch text NOT NULL,
 	field_parent varchar(64),
 	priority integer DEFAULT 5 NOT NULL,
-	primary key(id),
-	CONSTRAINT ir_ui_view_type CHECK (((("type")::text = ('form'::varchar)::text) OR (("type")::text = ('tree'::varchar)::text)))
+	primary key(id)
 );
 
 CREATE TABLE ir_ui_menu (
@@ -168,7 +174,7 @@ CREATE TABLE res_users (
 );
 alter table res_users add constraint res_users_login_uniq unique (login);
 
-insert into res_users (id,login,password,name,action_id,perm_id,active) values (1,'root',NULL,'Administrator',NULL,1,False);
+insert into res_users (id,login,password,name,action_id,perm_id,active) values (1,'root',NULL,'Root',NULL,1,False);
 select setval('res_users_id_seq', 2);
 
 CREATE TABLE res_groups (
@@ -319,6 +325,7 @@ CREATE TABLE ir_module_module (
     demo boolean default False,
     primary key(id)
 );
+ALTER TABLE ir_module_module add constraint name_uniq unique (name);
 
 CREATE TABLE ir_module_module_dependency (
     id serial NOT NULL,
@@ -333,3 +340,10 @@ CREATE TABLE ir_module_module_dependency (
     primary key(id)
 );
 
+CREATE TABLE res_company (
+    id serial NOT NULL,
+    perm_id integer,
+    name character varying(64) not null,
+    parent_id integer references res_company on delete set null,
+    primary key(id)
+);
