@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be)
 #
-# $Id: add_new.py 5002 2006-12-27 09:01:00Z ced $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -37,6 +37,8 @@ import tools
 import wizard
 import pooler
 
+import zipfile
+
 module_name_re = re.compile('.*addons.(.*?).__terp__.py$')
 
 _info_arch = '''<?xml version="1.0"?>
@@ -46,16 +48,6 @@ _info_arch = '''<?xml version="1.0"?>
 '''
 _info_fields = {}
 
-def get_module_info(name):
-	try:
-		f = file(os.path.join(tools.config['addons_path'], name, '__terp__.py'), 'r')
-		data = f.read()
-		info = eval(data)
-		f.close()
-	except:
-		return {}
-	return info
-
 class wizard_install_module(wizard.interface):
 	def watch_dir(self, cr, uid, data, context):
 		mod_obj = pooler.get_pool(cr.dbname).get('ir.module.module')
@@ -63,10 +55,13 @@ class wizard_install_module(wizard.interface):
 		known_modules = [x['name'] for x in all_mods]
 		ls_ad = glob.glob(os.path.join(tools.config['addons_path'], '*', '__terp__.py'))
 		modules = [module_name_re.match(name).group(1) for name in ls_ad]
+		for fname in os.listdir(tools.config['addons_path']):
+			if zipfile.is_zipfile(fname):
+				modules.append( fname.split('.')[0])
 		for module in modules:
 			if module in known_modules:
 				continue
-			terp = get_module_info(module)
+			terp = mod_obj.get_module_info(module)
 			if not terp.get('installable', True):
 				continue
 			imp.load_module(module, *imp.find_module(module))
@@ -80,7 +75,7 @@ class wizard_install_module(wizard.interface):
 			for d in dependencies:
 				cr.execute('insert into ir_module_module_dependency (module_id,name) values (%s, %s)', (mod_id, d))
 		for module in known_modules:
-			terp = get_module_info(module)
+			terp = mod_obj.get_module_info(module)
 			if terp.get('installable', True):
 				for mod in all_mods:
 					if mod['name'] == module and mod['state'] == 'uninstallable':
