@@ -34,7 +34,7 @@ import pooler
 
 view_form="""<?xml version="1.0"?>
 <form string="Import language">
-	<image name="gtk-info" size="64" colspan="2"/>
+	<image name="gtk-dialog-info" colspan="2"/>
 	<group colspan="2" col="4">
 		<separator string="Import new language" colspan="4"/>
 		<field name="name"/>
@@ -45,21 +45,29 @@ view_form="""<?xml version="1.0"?>
 	</group>
 </form>"""
 
+fields_form={
+	'name':{'string':'Language name', 'type':'char', 'size':64, 'required':True},
+	'code':{'string':'Code (eg:en__US)', 'type':'char', 'size':5, 'required':True},
+	'data':{'string':'File', 'type':'binary', 'required':True},
+}
+
 class wizard_import_lang(wizard.interface):
+
 	def _import_lang(self, cr, uid, data, context):
 		form=data['form']
 		buf=base64.decodestring(data['form']['data']).split('\n')
 		tools.trans_load_data(cr.dbname, buf, form['code'], lang_name=form['name'])
 		return {}
-	fields_form={
-		'name':{'string':'Language name', 'type':'char', 'size':64, 'required':True},
-		'code':{'string':'Code (eg:en_US)', 'type':'char', 'size':5, 'required':True},
-		'data':{'string':'File', 'type':'binary', 'required':True},
-	}
+
 	states={
 		'init':{
-			'actions':[],
-			'result':{'type':'form', 'arch':view_form, 'fields':fields_form, 'state':[('end', 'Cancel', 'gtk-cancel'), ('finish', 'Ok', 'gtk-ok')]}
+			'actions': [],
+			'result': {'type': 'form', 'arch': view_form, 'fields': fields_form,
+				'state':[
+					('end', 'Cancel', 'gtk-cancel'),
+					('finish', 'Ok', 'gtk-ok', True)
+				]
+			}
 		},
 		'finish':{
 			'actions':[],
