@@ -38,41 +38,12 @@ class res_partner_function(osv.osv):
 	_name = 'res.partner.function'
 	_description = 'Function of the contact'
 	_columns = {
-		'name': fields.char('Name', size=64, required=True),
+		'name': fields.char('Function name', size=64, required=True),
 		'code': fields.char('Code', size=8),
 	}
 	_order = 'name'
 res_partner_function()
 
-class res_country(osv.osv):
-	_name = 'res.country'
-	_description = 'Country'
-	_columns = {
-		'name': fields.char('Country Name', size=64, help='The full name of the country.'),
-		'code': fields.char('Country Code', size=2, help='The ISO country code in two chars. You can use this field for quick search.'),
-	}
-	_sql_constraints = [
-		('name_uniq', 'unique (name)', 'The name of the country must be unique !'),
-		('code_uniq', 'unique (code)', 'The code of the country must be unique !')
-	]
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('code','=',name)]+ args)
-		if not ids:
-			ids = self.search(cr, user, [('name',operator,name)]+ args)
-		return self.name_get(cr, user, ids)
-	_order='code'
-res_country()
-
-class res_country_state(osv.osv):
-	_description="Country state"
-	_name = 'res.country.state'
-	_columns = {
-		'country_id': fields.many2one('res.country', 'Country'),
-		'name': fields.char('State Name', size=64),
-		'code': fields.char('State Code', size=2),
-	}
-	_order = 'code'
-res_country_state()
 
 class res_payterm(osv.osv):
 	_description = 'Payment term'
@@ -82,21 +53,6 @@ class res_payterm(osv.osv):
 	}
 res_payterm()
 
-class res_partner_category_type(osv.osv):
-	_description='Partner category types'
-	_name = 'res.partner.category.type'
-	_columns = {
-		'name': fields.char('Category Name', required=True, size=64),
-	}
-	_order = 'name'
-res_partner_category_type()
-
-def _cat_type_get(self, cr, uid, context={}):
-	obj = self.pool.get('res.partner.category.type')
-	ids = obj.search(cr, uid, [])
-	res = obj.read(cr, uid, ids, ['name'], context)
-	return [(r['name'], r['name']) for r in res]
-
 class res_partner_category(osv.osv):
 	def name_get(self, cr, uid, ids, context={}):
 		if not len(ids):
@@ -127,7 +83,6 @@ class res_partner_category(osv.osv):
 	_name = 'res.partner.category'
 	_columns = {
 		'name': fields.char('Category Name', required=True, size=64),
-		'type_id': fields.many2one('res.partner.category.type', 'Category Type'),
 		'parent_id': fields.many2one('res.partner.category', 'Parent Category', select=True),
 		'complete_name': fields.function(_name_get_fnc, method=True, type="char", string='Name'),
 		'child_ids': fields.one2many('res.partner.category', 'parent_id', 'Childs Category'),
@@ -185,7 +140,7 @@ class res_partner(osv.osv):
 		'lang': fields.selection(_lang_get, 'Language', size=5),
 		'user_id': fields.many2one('res.users', 'Salesman'),
 		'responsible': fields.many2one('res.users', 'Users'),
-		'vat': fields.char('VAT',size=32),
+		'vat': fields.char('VAT',size=32 ,help="Value Added Tax number"),
 		'bank_ids': fields.one2many('res.partner.bank', 'partner_id', 'Banks'),
 		'website': fields.char('Website',size=64),
 		'comment': fields.text('Notes'),
@@ -202,6 +157,11 @@ class res_partner(osv.osv):
 	_sql_constraints = [
 		('name_uniq', 'unique (name)', 'The name of the partner must be unique !')
 	]
+
+	def copy(self, cr, uid, id, default=None, context={}):
+		name = self.read(cr, uid, [id], ['name'])[0]['name']
+		default.update({'name': name+' (copy)'})
+		return super(res_partner, self).copy(cr, uid, id, default, context)
 	
 	def _check_ean_key(self, cr, uid, ids):
 		for partner_o in pooler.get_pool(cr.dbname).get('res.partner').read(cr, uid, ids, ['ean13',]):
@@ -232,13 +192,17 @@ class res_partner(osv.osv):
 		res = [(r['id'], r[rec_name]) for r in self.read(cr, uid, ids, [rec_name], context)]
 		return res
 		
-	def name_search(self, cr, uid, name, args=[], operator='ilike', context={}, limit=80):
+	def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=80):
+		if not args:
+			args=[]
+		if not context:
+			context={}
 		if name:
-			ids = self.search(cr, uid, [('ref', '=', name)] + args, limit=limit)
+			ids = self.search(cr, uid, [('ref', '=', name)] + args, limit=limit, context=context)
 			if not ids:
-				ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit)
+				ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
 		else:
-			ids = self.search(cr, uid, args, limit=limit)
+			ids = self.search(cr, uid, args, limit=limit, context=context)
 		return self.name_get(cr, uid, ids, context)
 
 	def _email_send(self, cr, uid, ids, email_from, subject, body, on_error=None):
@@ -295,26 +259,6 @@ class res_partner(osv.osv):
 		return True
 res_partner()
 
-class res_partner_bank(osv.osv):
-	_description='Bank Details'
-	_name = "res.partner.bank"
-	_order = "sequence"
-	_columns = {
-		'name': fields.char('Account Name', size=64, required=True),
-		'sequence': fields.integer('Sequence'),
-		'iban': fields.char('Account number', size=64),
-		'swift': fields.char('Swift', size=64),
-		'bank_name': fields.char('Bank Name', size=64),
-		'bank_guichet': fields.char('Branch', size=64),
-		'partner_id': fields.many2one('res.partner', 'Partner', required=True, ondelete='cascade', select=True),
-		'active': fields.boolean('Active'),
-	}
-	_defaults = {
-		'active': lambda *a: 1,
-	}
-res_partner_bank()
-
-
 class res_partner_address(osv.osv):
 	_description ='Partner Contact'
 	_name = 'res.partner.address'
@@ -322,7 +266,7 @@ class res_partner_address(osv.osv):
 	_columns = {
 		'partner_id': fields.many2one('res.partner', 'Partner', required=True, ondelete='cascade', select=True),
 		'type': fields.selection( [ ('default','Default'),('invoice','Invoice'), ('delivery','Delivery'), ('contact','Contact'), ('other','Other') ],'Address Type'),
-		'function': fields.many2one('res.partner.function', 'Function', relate=True),
+		'function': fields.many2one('res.partner.function', 'Function'),
 		'title': fields.selection(_contact_title_get, 'Title', size=32),
 		'name': fields.char('Contact Name', size=64),
 		'street': fields.char('Street', size=128),
@@ -354,19 +298,142 @@ class res_partner_address(osv.osv):
 				if r['name'] and (r['zip'] or r['city']):
 					addr += ', '
 				addr += str(r['zip'] or '') + ' ' + str(r['city'] or '')
-				res.append((r['id'], addr))
+				res.append((r['id'], addr or '/'))
 		return res
 
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}, limit=80):
+	def name_search(self, cr, user, name, args=None, operator='ilike', context=None, limit=80):
+		if not args:
+			args=[]
+		if not context:
+			context={}
 		if context.get('contact_display', 'contact')=='partner':
-			ids = self.search(cr, user, [('partner_id',operator,name)], limit=limit)
+			ids = self.search(cr, user, [('partner_id',operator,name)], limit=limit, context=context)
 		else:
-			ids = self.search(cr, user, [('zip','=',name)] + args, limit=limit)
+			ids = self.search(cr, user, [('zip','=',name)] + args, limit=limit, context=context)
 			if not ids: 
-				ids = self.search(cr, user, [('city',operator,name)] + args, limit=limit)
-			if not ids:
-				ids = self.search(cr, user, [('name',operator,name)] + args, limit=limit)
-		return self.name_get(cr, user, ids)
+				ids = self.search(cr, user, [('city',operator,name)] + args, limit=limit, context=context)
+			if name:
+				ids += self.search(cr, user, [('name',operator,name)] + args, limit=limit, context=context)
+				ids += self.search(cr, user, [('partner_id',operator,name)] + args, limit=limit, context=context)
+		return self.name_get(cr, user, ids, context=context)
 res_partner_address()
 
+class res_partner_bank_type(osv.osv):
+	_description='Bank Account Type'
+	_name = 'res.partner.bank.type'
+	_columns = {
+		'name': fields.char('Name', size=64, required=True),
+		'code': fields.char('Code', size=64, required=True),
+		'field_ids': fields.one2many('res.partner.bank.type.field', 'bank_type_id', 'Type fields'),
+	}
+res_partner_bank_type()
+
+class res_partner_bank_type_fields(osv.osv):
+	_description='Bank type fields'
+	_name = 'res.partner.bank.type.field'
+	_columns = {
+		'name': fields.char('Field name', size=64, required=True),
+		'bank_type_id': fields.many2one('res.partner.bank.type', 'Bank type', required=True, ondelete='cascade'),
+		'required': fields.boolean('Required'),
+		'readonly': fields.boolean('Readonly'),
+		'size': fields.integer('Max. Size'),
+	}
+res_partner_bank_type_fields()
+
+
+class res_partner_bank(osv.osv):
+	'''Bank Accounts'''
+	_name = "res.partner.bank"
+	_rec_name = "state"
+	_description = __doc__
+	_order = 'sequence'
+
+	def _bank_type_get(self, cr, uid, context=None):
+		bank_type_obj = self.pool.get('res.partner.bank.type')
+
+		result = []
+		type_ids = bank_type_obj.search(cr, uid, [])
+		bank_types = bank_type_obj.browse(cr, uid, type_ids)
+		for bank_type in bank_types:
+			result.append((bank_type.code, bank_type.name))
+		return result
+
+	def _default_value(self, cursor, user, field, context=None):
+		if field in ('country_id', 'state_id'):
+			value = False
+		else:
+			value = ''
+		if not context.get('address', False):
+			return value
+		for ham, spam, address in context['address']:
+			if address['type'] == 'default':
+				return address[field]
+			elif not address['type']:
+				value = address[field]
+		return value
+
+	_columns = {
+		'name': fields.char('Description', size=128),
+		'acc_number': fields.char('Account number', size=64, required=False),
+		'bank': fields.many2one('res.bank', 'Bank'),
+		'owner_name': fields.char('Account owner', size=64),
+		'street': fields.char('Street', size=128),
+		'zip': fields.char('Zip', change_default=True, size=24),
+		'city': fields.char('City', size=128),
+		'country_id': fields.many2one('res.country', 'Country'),
+		'partner_id': fields.many2one('res.partner', 'Partner', required=True,
+			ondelete='cascade', select=True),
+		'state': fields.selection(_bank_type_get, 'Bank type', required=True),
+		'sequence': fields.integer('Sequence'),
+		'state_id': fields.many2one('res.country.state', 'State',
+			domain="[('country_id', '=', country_id)]"),
+	}
+	_defaults = {
+		'owner_name': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'name', context=context),
+		'street': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'street', context=context),
+		'city': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'city', context=context),
+		'zip': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'zip', context=context),
+		'country_id': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'country_id', context=context),
+		'state_id': lambda obj, cursor, user, context: obj._default_value(
+			cursor, user, 'state_id', context=context),
+	}
+
+	def fields_get(self, cr, uid, fields=None, context=None):
+		res = super(res_partner_bank, self).fields_get(cr, uid, fields, context)
+		bank_type_obj = self.pool.get('res.partner.bank.type')
+		type_ids = bank_type_obj.search(cr, uid, [])
+		types = bank_type_obj.browse(cr, uid, type_ids)
+		for type in types:
+			for field in type.field_ids:
+				if field.name in res:
+					res[field.name].setdefault('states', {})
+					res[field.name]['states'][type.code] = [
+							('readonly', field.readonly),
+							('required', field.required)]
+		return res
+
+	def name_get(self, cr, uid, ids, context=None):
+		if not len(ids):
+			return []
+		bank_type_obj = self.pool.get('res.partner.bank.type')
+
+		type_ids = bank_type_obj.search(cr, uid, [])
+		bank_type_names = {}
+		for bank_type in bank_type_obj.browse(cr, uid, type_ids,
+				context=context):
+			bank_type_names[bank_type.code] = bank_type.name
+
+		return [(r['id'], (bank_type_names[r['state']] or '') + (r['owner_name'] and \
+				(': ' + r['owner_name']) or '')) for r in self.read(cr, uid, ids,
+					[self._rec_name, 'owner_name'], context,
+					load='_classic_write')]
+
+res_partner_bank()
+
+
 # vim:noexpandtab:
