@@ -26,5 +26,8 @@
 #
 ##############################################################################
 
-from crm import *
+import crm
+import crm_segmentation
+
 import report
+import wizard
