@@ -30,6 +30,7 @@ import tools
 from osv import fields,osv,orm
 
 import mx.DateTime
+import base64
 
 MAX_LEVEL = 15
 AVAILABLE_STATES = [
@@ -67,6 +68,19 @@ class crm_case_section(osv.osv):
 	_sql_constraints = [
 		('code_uniq', 'unique (code)', 'The code of the section must be unique !')
 	]
+	def _check_recursion(self, cr, uid, ids):
+		level = 100
+		while len(ids):
+			cr.execute('select distinct parent_id from crm_case_section where id in ('+','.join(map(str,ids))+')')
+			ids = filter(None, map(lambda x:x[0], cr.fetchall()))
+			if not level:
+				return False
+			level -= 1
+		return True
+	_constraints = [
+		(_check_recursion, 'Error ! You can not create recursive sections.', ['parent_id'])
+	]
+
 	def menu_create(self, cr, uid, ids, name, menu_parent_id=False, context={}):
 		menus = {}
 		menus[-1] = menu_parent_id
@@ -74,21 +88,22 @@ class crm_case_section(osv.osv):
 			for (index, mname, mdomain, latest) in [
 				(0,'',"[('section_id','=',"+str(section.id)+")]", -1),
 				(1,'My ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid)]", 0),
-				(2,'My Unclosed ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','<>','cancel'), ('state','<>','close')]", 1),
+				(2,'My Unclosed ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','<>','cancel'), ('state','<>','done')]", 1),
 				(5,'My Open ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','=','open')]", 2),
 				(6,'My Pending ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','=','pending')]", 2),
 				(7,'My Draft ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','=','draft')]", 2),
 
-				(3,'My Late ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','close')]", 1),
+				(3,'My Late ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','done')]", 1),
 				(4,'My Canceled ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('state','=','cancel')]", 1),
 				(8,'All ',"[('section_id','=',"+str(section.id)+"),]", 0),
 				(9,'Unassigned ',"[('section_id','=',"+str(section.id)+"),('user_id','=',False)]", 8),
-				(10,'Late ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','close')]", 8),
+				(10,'Late ',"[('section_id','=',"+str(section.id)+"),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','done')]", 8),
 				(11,'Canceled ',"[('section_id','=',"+str(section.id)+"),('state','=','cancel')]", 8),
-				(12,'Unclosed ',"[('section_id','=',"+str(section.id)+"),('state','<>','cancel'), ('state','<>','close')]", 8),
+				(12,'Unclosed ',"[('section_id','=',"+str(section.id)+"),('state','<>','cancel'), ('state','<>','done')]", 8),
 				(13,'Open ',"[('section_id','=',"+str(section.id)+"),('state','=','open')]", 12),
 				(14,'Pending ',"[('section_id','=',"+str(section.id)+"),('state','=','pending')]", 12),
 				(15,'Draft ',"[('section_id','=',"+str(section.id)+"),('state','=','draft')]", 12),
+				(16,'Unassigned ',"[('section_id','=',"+str(section.id)+"),('user_id','=',False),('state','<>','cancel'),('state','<>','done')]", 12),
 			]:
 				view_mode = 'tree,form'
 				icon = 'STOCK_JUSTIFY_FILL'
@@ -102,7 +117,7 @@ class crm_case_section(osv.osv):
 				})
 				menus[index] = menu_id
 				action_id = self.pool.get('ir.actions.act_window').create(cr,uid, {
-					'name': 'Action '+mname+name+' Cases',
+					'name': mname+name+' Cases',
 					'res_model': 'crm.case',
 					'domain': mdomain,
 					'view_type': 'form',
@@ -135,8 +150,12 @@ class crm_case_categ(osv.osv):
 	_description = "Category of case"
 	_columns = {
 		'name': fields.char('Case Category Name', size=64, required=True),
+		'probability': fields.float('Probability', required=True),
 		'section_id': fields.many2one('crm.case.section', 'Case Section'),
 	}
+	_defaults = {
+		'probability': lambda *args: 0.0
+	}
 crm_case_categ()
 
 class crm_case_rule(osv.osv):
@@ -147,8 +166,8 @@ class crm_case_rule(osv.osv):
 		'active': fields.boolean('Active'),
 		'sequence': fields.integer('Sequence'),
 
-		'trg_state_from': fields.selection([('','')]+AVAILABLE_STATES, 'Case state from', size=16),
-		'trg_state_to': fields.selection([('','')]+AVAILABLE_STATES, 'Case state to', size=16),
+		'trg_state_from': fields.selection([('',''),('escalate','Escalate')]+AVAILABLE_STATES, 'Case state from', size=16),
+		'trg_state_to': fields.selection([('',''),('escalate','Escalate')]+AVAILABLE_STATES, 'Case state to', size=16),
 
 		'trg_date_type':  fields.selection([
 			('none','None'),
@@ -157,7 +176,7 @@ class crm_case_rule(osv.osv):
 			('deadline','Deadline'),
 			], 'Trigger Date', size=16),
 		'trg_date_range': fields.integer('Delay after trigger date'),
-		'trg_date_range_type': fields.selection([('hour','Hours'),('day','Days'),('month','Months')], 'Delay type'),
+		'trg_date_range_type': fields.selection([('minutes', 'Minutes'),('hour','Hours'),('day','Days'),('month','Months')], 'Delay type'),
 
 		'trg_section_id': fields.many2one('crm.case.section', 'Section'),
 		'trg_categ_id':  fields.many2one('crm.case.categ', 'Category', domain="[('section_id','=',trg_section_id)]"),
@@ -178,6 +197,7 @@ class crm_case_rule(osv.osv):
 
 		'act_remind_partner': fields.boolean('Remind partner'),
 		'act_remind_user': fields.boolean('Remind responsible'),
+		'act_remind_attach': fields.boolean('Remind with attachment'),
 
 		'act_mail_to_user': fields.boolean('Mail to responsible'),
 		'act_mail_to_partner': fields.boolean('Mail to partner'),
@@ -196,12 +216,18 @@ class crm_case_rule(osv.osv):
 		'act_mail_to_watchers': lambda *a: 0,
 	}
 	_order = 'sequence'
-	#
-	# Function called by the sceduler to proccess cases for date actions
-	# Only works on not done and canceled cases
-	#
+
 	def _check(self, cr, uid, ids=False, context={}):
-		cr.execute('select * from crm_case where (date_action_last<%s or date_action_last is null) and (date_action_next<=%s or date_action_next is null) and state not in (\'cancel\',\'done\')', (time.strftime("%Y-%m-%d %H:%M:%S"), time.strftime('%Y-%m-%d %H:%M:%S')) )
+		'''
+		Function called by the sceduler to proccess cases for date actions
+		Only works on not done and canceled cases
+		'''
+		cr.execute('select * from crm_case \
+				where (date_action_last<%s or date_action_last is null) \
+				and (date_action_next<=%s or date_action_next is null) \
+				and state not in (\'cancel\',\'done\')',
+				(time.strftime("%Y-%m-%d %H:%M:%S"),
+					time.strftime('%Y-%m-%d %H:%M:%S')))
 		ids2 = map(lambda x: x[0], cr.fetchall() or [])
 		case_obj = self.pool.get('crm.case')
 		cases = case_obj.browse(cr, uid, ids2, context)
@@ -217,46 +243,61 @@ def _links_get(self, cr, uid, context={}):
 class crm_case(osv.osv):
 	_name = "crm.case"
 	_description = "Case"
+
+	def _email_last(self, cursor, user, ids, name, arg, context=None):
+		res = {}
+		for case in self.browse(cursor, user, ids):
+			if case.history_line:
+				res[case.id] = case.history_line[0].description
+			else:
+				res[case.id] = False
+		return res
+
 	_columns = {
-		'id': fields.integer('Case ID', readonly=True),
-		'name': fields.char('Case Description',size=64, required=True),
+		'id': fields.integer('ID', readonly=True),
+		'name': fields.char('Description',size=64, required=True),
 		'priority': fields.selection(AVAILABLE_PRIORITIES, 'Priority'),
 		'active': fields.boolean('Active'),
-		'description': fields.text('Description'),
-		'section_id': fields.many2one('crm.case.section', 'Case Section', required=True),
-		'categ_id': fields.many2one('crm.case.categ', 'Category', domain="[('section_id','=',section_id)]", relate=True),
+		'description': fields.text('Your action'),
+		'section_id': fields.many2one('crm.case.section', 'Section', required=True, select=True),
+		'categ_id': fields.many2one('crm.case.categ', 'Category', domain="[('section_id','=',section_id)]"),
 		'planned_revenue': fields.float('Planned Revenue'),
 		'planned_cost': fields.float('Planned Costs'),
 		'probability': fields.float('Probability (0.50)'),
 		'email_from': fields.char('Partner Email', size=128),
 		'email_cc': fields.char('Watchers Emails', size=252),
-		'email_last': fields.text('Latest E-Mail'),
-		'partner_id': fields.many2one('res.partner', 'Partner', relate=True),
+		'email_last': fields.function(_email_last, method=True,
+			string='Latest E-Mail', type='text'),
+		'partner_id': fields.many2one('res.partner', 'Partner'),
 		'partner_address_id': fields.many2one('res.partner.address', 'Partner Contact', domain="[('partner_id','=',partner_id)]"),
 		'som': fields.many2one('res.partner.som', 'State of Mind'),
 		'date': fields.datetime('Date'),
-		'create_date': fields.datetime('Date Created'),
-		'date_deadline': fields.date('Date Deadline'),
-		'date_closed': fields.datetime('Date Closed', readonly=True),
-		'canal_id': fields.many2one('res.partner.canal', 'Channel', relate=True),
-		'user_id': fields.many2one('res.users', 'User Responsible', relate=True),
-		'history_line': fields.one2many('crm.case.history', 'case_id', 'Case History'),
+		'create_date': fields.datetime('Created' ,readonly=True),
+		'date_deadline': fields.date('Deadline'),
+		'date_closed': fields.datetime('Closed', readonly=True),
+		'canal_id': fields.many2one('res.partner.canal', 'Channel'),
+		'user_id': fields.many2one('res.users', 'User Responsible'),
+		'history_line': fields.one2many('crm.case.history', 'case_id', 'Communication'),
+		'log_ids': fields.one2many('crm.case.log', 'case_id', 'Logs History'),
 		'state': fields.selection(AVAILABLE_STATES, 'State', size=16, readonly=True),
 		'ref' : fields.reference('Reference', selection=_links_get, size=128),
 		'ref2' : fields.reference('Reference 2', selection=_links_get, size=128),
 
-		'date_action_last': fields.datetime('Date Last Action', readonly=1),
-		'date_action_next': fields.datetime('Date Next Action', readonly=1),
+		'date_action_last': fields.datetime('Last Action', readonly=1),
+		'date_action_next': fields.datetime('Next Action', readonly=1),
 	}
 	_defaults = {
 		'active': lambda *a: 1,
 		'user_id': lambda s,cr,uid,c={}: uid,
 		'state': lambda *a: 'draft',
-		'priority': lambda *a: AVAILABLE_PRIORITIES[1][0],
+		'priority': lambda *a: AVAILABLE_PRIORITIES[2][0],
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 	}
-	_order = 'priority, date_deadline, id'
-	def _action(self, cr, uid, cases, state_to, scrit=[], context={}):
+	_order = 'priority, date_deadline desc, id desc'
+
+	def _action(self, cr, uid, cases, state_to, scrit=None, context={}):
+		if not scrit:
+			scrit = []
 		action_ids = self.pool.get('crm.case.rule').search(cr, uid, scrit)
 		level = MAX_LEVEL
 		while len(action_ids) and level:
@@ -278,8 +319,8 @@ class crm_case(osv.osv):
 							(action.trg_partner_categ_id.id in map(lambda x: x.id, case.partner_id.category_id or []))
 						)
 					)
-					ok = ok and (not action.trg_priority_from or action.trg_priority_from<=case.priority)
-					ok = ok and (not action.trg_priority_to or action.trg_priority_to>=case.priority)
+					ok = ok and (not action.trg_priority_from or action.trg_priority_from>=case.priority)
+					ok = ok and (not action.trg_priority_to or action.trg_priority_to<=case.priority)
 					if not ok:
 						continue
 
@@ -291,19 +332,23 @@ class crm_case(osv.osv):
 							base = mx.DateTime.strptime(case.date_action_last, '%Y-%m-%d %H:%M:%S')
 						else:
 							base = mx.DateTime.strptime(case.create_date[:19], '%Y-%m-%d %H:%M:%S')
-					elif action.trg_date_type=='deadline':
+					elif action.trg_date_type=='deadline' and case.date_deadline:
 						base = mx.DateTime.strptime(case.date_deadline, '%Y-%m-%d')
 					if base:
 						fnct = {
+							'minutes': lambda interval: mx.DateTime.RelativeDateTime(minutes=interval),
 							'day': lambda interval: mx.DateTime.RelativeDateTime(days=interval),
 							'hour': lambda interval: mx.DateTime.RelativeDateTime(hours=interval),
 							'month': lambda interval: mx.DateTime.RelativeDateTime(months=interval),
 						}
 						d = base + fnct[action.trg_date_range_type](action.trg_date_range)
 						dt = d.strftime('%Y-%m-%d %H:%M:%S')
-						ok = (dt <= time.strftime('%Y-%m-%d %H:%M:%S')) and ((not case.date_action_next) or dt>case.date_action_next)
+						ok = (dt <= time.strftime('%Y-%m-%d %H:%M:%S')) and \
+								((not case.date_action_next) or \
+								(dt >= case.date_action_next and \
+								case.date_action_last < case.date_action_next))
 						if not ok:
-							if not case.date_action_next or dt<case.date_action_next:
+							if not case.date_action_next or dt < case.date_action_next:
 								case.date_action_next = dt
 								self.write(cr, uid, [case.id], {'date_action_next': dt}, context)
 
@@ -313,12 +358,16 @@ class crm_case(osv.osv):
 					if ok:
 						write = {}
 						if action.act_state:
+							case.state = action.act_state
 							write['state'] = action.act_state
 						if action.act_section_id:
+							case.section_id = action.act_section_id
 							write['section_id'] = action.act_section_id.id
 						if action.act_user_id:
+							case.user_id = action.act_user_id
 							write['user_id'] = action.act_user_id.id
 						if action.act_priority:
+							case.priority = action.act_priority
 							write['priority'] = action.act_priority
 						if action.act_email_cc:
 							if '@' in (case.email_cc or ''):
@@ -329,9 +378,9 @@ class crm_case(osv.osv):
 						self.write(cr, uid, [case.id], write, context)
 						caseobj = self.pool.get('crm.case')
 						if action.act_remind_user:
-							caseobj.remind_user(cr, uid, [case.id], context)
+							caseobj.remind_user(cr, uid, [case.id], context, attach=action.act_remind_attach)
 						if action.act_remind_partner:
-							caseobj.remind_partner(cr, uid, [case.id], context)
+							caseobj.remind_partner(cr, uid, [case.id], context, attach=action.act_remind_attach)
 						if action.act_method:
 							getattr(caseobj, 'act_method')(cr, uid, [case.id], action, context)
 						emails = []
@@ -345,7 +394,7 @@ class crm_case(osv.osv):
 						if action.act_mail_to_email:
 							emails += (action.act_mail_to_email or '').split(',')
 						emails = filter(None, emails)
-						if len(emails):
+						if len(emails) and action.act_mail_body:
 							self.email_send(cr, uid, case, emails, action.act_mail_body)
 						break
 			action_ids = newactions
@@ -359,15 +408,19 @@ class crm_case(osv.osv):
 			'case_date': case.date,
 
 			'case_user': case.user_id.name or '/',
-			'case_user_email': case.user_id.address_id.email,
-			'case_user_phone': case.user_id.address_id.phone,
+			'case_user_email': (case.user_id and case.user_id.address_id and case.user_id.address_id.email) or '/',
+			'case_user_phone': (case.user_id and case.user_id.address_id and case.user_id.address_id.phone) or '/',
 
 			'email_from': case.email_from,
 			'partner': (case.partner_id and case.partner_id.name) or '/',
 			'partner_email': (case.partner_address_id and case.partner_address_id.email) or '/',
 		}
 		body = body % data
-		tools.email_send(case.user_id.address_id.email, emails, '['+str(case.id)+'] '+case.name, body, reply_to=case.section_id.reply_to)
+		if case.user_id and case.user_id.address_id and case.user_id.address_id.email:
+			emailfrom = case.user_id.address_id.email
+		else:
+			emailfrom = case.section_id.reply_to
+		tools.email_send(emailfrom, emails, '['+str(case.id)+'] '+case.name, body, reply_to=case.section_id.reply_to, tinycrm=str(case.id))
 		return True
 	def __log(self, cr, uid, cases, keyword, context={}):
 		if not self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_'+keyword):
@@ -385,63 +438,107 @@ class crm_case(osv.osv):
 				})
 		return True
 
-	def __history(self, cr, uid, cases, keyword, context={}):
+	def __history(self, cr, uid, cases, keyword, history=False, email=False, context={}):
 		for case in cases:
-			self.pool.get('crm.case.history').create(cr, uid, {
+			data = {
 				'name': keyword,
-				'description': case.description,
-				'som': False,
-				'canal_id': False,
+				'som': case.som.id,
+				'canal_id': case.canal_id.id,
 				'user_id': uid,
 				'case_id': case.id
-			})
+			}
+			obj = self.pool.get('crm.case.log')
+			if history and case.description:
+				obj = self.pool.get('crm.case.history')
+				data['description'] = case.description
+				data['email'] = email or \
+						(case.user_id and case.user_id.address_id and \
+							case.user_id.address_id.email) or False
+			obj.create(cr, uid, data, context)
 		return True
 
 	def create(self, cr, uid, *args, **argv):
 		res = super(crm_case, self).create(cr, uid, *args, **argv)
 		cases = self.browse(cr, uid, [res])
+		cases[0].state # to fill the browse record cache
 		self.__log(cr,uid, cases, 'draft')
 		self._action(cr,uid, cases, 'draft')
 		return res
 
-	def remind_partner(self, cr, uid, ids, *args):
+	def remind_partner(self, cr, uid, ids, context={}, attach=False):
+		return self.remind_user(cr, uid, ids, context, attach,
+				destination=False)
+
+	def remind_user(self, cr, uid, ids, context={}, attach=False,
+			destination=True):
 		for case in self.browse(cr, uid, ids):
-			if case.user_id and case.user_id.address_id and case.user_id.address_id.email and case.email_from:
-				tools.email_send(
-					case.user_id.address_id.email,
-					[case.email_from],
-					'Reminder: '+'['+str(case.id)+']'+' '+case.name,
-					case.email_last or case.description, reply_to=case.section_id.reply_to
-					)
+			if case.user_id and case.user_id.address_id \
+					and case.user_id.address_id.email \
+					and case.email_from:
+				src = case.email_from
+				dest = case.user_id.address_id.email
+				if not destination:
+					src,dest = dest,src
+				dest = [dest]
+				if not attach:
+					tools.email_send(
+						src,
+						dest,
+						'Reminder: '+'['+str(case.id)+']'+' '+case.name,
+						case.email_last or case.description,
+						reply_to=case.section_id.reply_to, tinycrm=str(case.id)
+						)
+				else:
+					attach_ids = self.pool.get('ir.attachment').search(cr, uid,
+							[('res_model', '=', 'crm.case'),
+								('res_id', '=', case.id)])
+					res = self.pool.get('ir.attachment').read(cr, uid,
+							attach_ids, ['datas_fname','datas'])
+					res = map(lambda x: (x['datas_fname'],
+						base64.decodestring(x['datas'])), res)
+					tools.email_send_attach(
+						src,
+						dest,
+						'Reminder: '+'['+str(case.id)+']'+' '+case.name,
+						case.email_last or case.description,
+						reply_to=case.section_id.reply_to,
+						attach=res, tinycrm=str(case.id)
+						)
 		return True
 
-	def remind_user(self, cr, uid, ids, *args):
-		for case in self.browse(cr, uid, ids):
-			if case.user_id and case.user_id.address_id and case.user_id.address_id.email and case.email_from:
-				tools.email_send(
-					case.email_from,
-					[case.user_id.address_id.email],
-					'Reminder: '+'['+str(case.id)+']'+' '+case.name,
-					case.email_last or case.description, reply_to=case.section_id.reply_to
-					)
+	def add_reply(self, cursor, user, ids, context=None):
+		for case in self.browse(cursor, user, ids, context=context):
+			if case.history_line:
+				description = case.history_line[0].description
+				self.write(cursor, user, case.id, {
+					'description': '> ' + description.replace('\n','\n> '),
+					}, context=context)
 		return True
 
-	def case_log(self, cr, uid, ids, *args):
+	def case_log(self, cr, uid, ids,context={}, email=False, *args):
 		cases = self.browse(cr, uid, ids)
-		self.__history(cr, uid, cases, '')
-		return self.write(cr, uid, ids, {'description':False, 'som':False, 'canal_id': False})
+		self.__history(cr, uid, cases, 'Historize', history=True, email=email)
+		return self.write(cr, uid, ids, {'description': False, 'som': False,
+			'canal_id': False})
 
-	def case_log_reply(self, cr, uid, ids, *args):
+	def case_log_reply(self, cr, uid, ids, context={}, email=False, *args):
 		cases = self.browse(cr, uid, ids)
 		for case in cases:
 			if not case.email_from:
-				raise osv.except_osv('Error !', 'You must put a Partner eMail to use this action !')
-		self.__history(cr, uid, cases, '')
+				raise osv.except_osv('Error!',
+						'You must put a Partner eMail to use this action!')
+		self.__history(cr, uid, cases, 'Send', history=True, email=False)
 		for case in cases:
-			self.write(cr, uid, [case.id], {'description':False, 'som':False, 'canal_id': False, 'email_last':case.description})
+			self.write(cr, uid, [case.id], {
+				'description': False,
+				'som': False,
+				'canal_id': False,
+				})
 			emails = [case.email_from] + (case.email_cc or '').split(',')
 			emails = filter(None, emails)
-			tools.email_send(case.user_id.address_id.email, emails, '['+str(case.id)+'] '+case.name, case.description, reply_to=case.section_id.reply_to)
+			tools.email_send(case.user_id.address_id.email, emails,
+					'['+str(case.id)+'] '+case.name, case.description,
+					reply_to=case.section_id.reply_to, tinycrm=str(case.id))
 		return True
 
 	def onchange_partner_id(self, cr, uid, ids, part, email=False):
@@ -453,6 +550,13 @@ class crm_case(osv.osv):
 			data['email_from'] = self.pool.get('res.partner.address').browse(cr, uid, addr['contact']).email
 		return {'value':data}
 
+	def onchange_categ_id(self, cr, uid, ids, categ, context={}):
+		if not categ:
+			return {'value':{}}
+		cat = self.pool.get('crm.case.categ').browse(cr, uid, categ, context).probability
+		return {'value':{'probability':cat}}
+
+
 	def onchange_partner_address_id(self, cr, uid, ids, part, email=False):
 		if not part:
 			return {'value':{}}
@@ -463,7 +567,8 @@ class crm_case(osv.osv):
 
 	def case_close(self, cr, uid, ids, *args):
 		cases = self.browse(cr, uid, ids)
-		self.__log(cr,uid, cases, 'close')
+		cases[0].state # to fill the browse record cache
+		self.__log(cr,uid, cases, 'done')
 		self.__history(cr, uid, cases, 'Close')
 		self.write(cr, uid, ids, {'state':'done', 'date_closed': time.strftime('%Y-%m-%d %H:%M:%S')})
 		#
@@ -472,6 +577,22 @@ class crm_case(osv.osv):
 		self._action(cr,uid, cases, 'done')
 		return True
 
+	def case_escalate(self, cr, uid, ids, *args):
+		cases = self.browse(cr, uid, ids)
+		for case in cases:
+			data = {'active':True, 'user_id': False}
+			if case.section_id.parent_id:
+				data['section_id'] = case.section_id.parent_id.id
+				if case.section_id.parent_id.user_id:
+					data['user_id'] = case.section_id.parent_id.user_id.id
+			else:
+				raise osv.except_osv('Error !', 'You can not escalate this case.\nYou are already at the top level.')
+			self.write(cr, uid, ids, data)
+		self.__history(cr, uid, cases, 'escalate')
+		self._action(cr, uid, cases, 'escalate')
+		return True
+
+
 	def case_open(self, cr, uid, ids, *args):
 		cases = self.browse(cr, uid, ids)
 		self.__log(cr, uid, cases, 'open')
@@ -490,6 +611,7 @@ class crm_case(osv.osv):
 
 	def case_cancel(self, cr, uid, ids, *args):
 		cases = self.browse(cr, uid, ids)
+		cases[0].state # to fill the browse record cache
 		self.__log(cr, uid, cases, 'cancel')
 		self.__history(cr, uid, cases, 'Cancel')
 		self.write(cr, uid, ids, {'state':'cancel', 'active':True})
@@ -498,6 +620,7 @@ class crm_case(osv.osv):
 
 	def case_pending(self, cr, uid, ids, *args):
 		cases = self.browse(cr, uid, ids)
+		cases[0].state # to fill the browse record cache
 		self.__log(cr, uid, cases, 'pending')
 		self.__history(cr, uid, cases, 'Pending')
 		self.write(cr, uid, ids, {'state':'pending', 'active':True})
@@ -506,6 +629,7 @@ class crm_case(osv.osv):
 
 	def case_reset(self, cr, uid, ids, *args):
 		cases = self.browse(cr, uid, ids)
+		cases[0].state # to fill the browse record cache
 		self.__log(cr, uid, cases, 'draft')
 		self.__history(cr, uid, cases, 'Draft')
 		self.write(cr, uid, ids, {'state':'draft', 'active':True})
@@ -513,20 +637,32 @@ class crm_case(osv.osv):
 		return True
 crm_case()
 
-class crm_case_history(osv.osv):
-	_name = "crm.case.history"
-	_description = "Case history"
+class crm_case_log(osv.osv):
+	_name = "crm.case.log"
+	_description = "Case communication history"
+	_order = "id desc"
 	_columns = {
-		'name': fields.char('Name', size=64),
-		'description': fields.text('Description'),
+		'name': fields.char('Action', size=64),
 		'som': fields.many2one('res.partner.som', 'State of Mind'),
-		'date': fields.date('Date'),
+		'date': fields.datetime('Date'),
 		'canal_id': fields.many2one('res.partner.canal', 'Channel'),
 		'user_id': fields.many2one('res.users', 'User Responsible', readonly=True),
 		'case_id': fields.many2one('crm.case', 'Case', required=True, ondelete='cascade')
 	}
 	_defaults = {
-		'date': lambda *a: time.strftime('%Y-%m-%d'),
+		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
+	}
+crm_case_log()
+
+class crm_case_history(osv.osv):
+	_name = "crm.case.history"
+	_description = "Case history"
+	_order = "id desc"
+	_inherits = {'crm.case.log':"log_id"}
+	_columns = {
+		'description': fields.text('Description'),
+		'email': fields.char('Email', size=84),
+		'log_id': fields.many2one('crm.case.log','Log'),
 	}
 crm_case_history()
 
