@@ -29,5 +29,7 @@
 import delivery
 import partner
 import wizard
+import sale
+import stock
 
 #import report
