@@ -26,6 +26,7 @@
 #
 ##############################################################################
 
+import netsvc
 from osv import fields,osv,orm
 
 class delivery_carrier(osv.osv):
@@ -34,7 +35,7 @@ class delivery_carrier(osv.osv):
 	_columns = {
 		'name': fields.char('Carrier', size=64, required=True),
 		'partner_id': fields.many2one('res.partner', 'Carrier partner', required=True),
-		'product_id': fields.many2one('product.product', 'Delivery product', required=True, domain=[('type','=','service')]),
+		'product_id': fields.many2one('product.product', 'Delivery product', required=True),
 		'grids_id': fields.one2many('delivery.grid', 'carrier_id', 'Delivery grids'),
 		'active': fields.boolean('Active')
 	}
@@ -79,6 +80,47 @@ class delivery_grid(osv.osv):
 		'sequence': lambda *a: 1,
 	}
 	_order = 'sequence'
+
+
+	def get_price(self, cr, uid, id, order, dt, context):
+
+		total = 0
+		weight = 0
+		volume = 0
+
+		for line in order.order_line:
+			if not line.product_id:
+				continue
+			total += line.price_subtotal or 0.0
+			weight += (line.product_id.weight or 0.0) * line.product_uom_qty
+			volume += (line.product_id.volume or 0.0) * line.product_uom_qty
+
+
+		return self.get_price_from_picking(cr, uid, id, total,weight, volume, context)
+
+			
+	def get_price_from_picking(self, cr, uid, id, total, weight, volume, context={}):
+		grid = self.browse(cr, uid, id, context)
+
+		price = 0.0
+		ok = False
+
+		for line in grid.line_ids:
+			price_dict = {'price': total, 'volume':volume, 'weight': weight, 'wv':volume*weight}
+			test = eval(line.type+line.operator+str(line.max_value), price_dict)
+			if test:
+				if line.price_type=='variable':
+					price = line.list_price * price_dict[line.variable_factor]
+				else:
+					price = line.list_price
+				ok = True
+				break
+		if not ok:
+			raise except_osv('No price avaible !', 'No line matched this order in the choosed delivery grids !')
+
+		return price
+
+
 delivery_grid()
 
 class delivery_grid_line(osv.osv):
@@ -102,5 +144,8 @@ class delivery_grid_line(osv.osv):
 		'variable_factor': lambda *args: 'weight',
 	}
 	_order = 'list_price'
+
+
 delivery_grid_line()
 
+
