@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: hr.py 4656 2006-11-24 09:58:42Z ced $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -94,12 +94,33 @@ hr_employee_category()
 class hr_employee(osv.osv):
 	_name = "hr.employee"
 	_description = "Employee"
+
+	def _state(self, cr, uid, ids, name, args, context={}):
+		result = {}
+		for id in ids:
+			result[id] = 'absent'
+		cr.execute('SELECT hr_attendance.action, hr_attendance.employee_id \
+				FROM ( \
+					SELECT MAX(name) AS name, employee_id \
+					FROM hr_attendance \
+					WHERE action in (\'sign_in\', \'sign_out\') \
+					GROUP BY employee_id \
+				) AS foo \
+				LEFT JOIN hr_attendance \
+					ON (hr_attendance.employee_id = foo.employee_id \
+						AND hr_attendance.name = foo.name) \
+				WHERE hr_attendance.employee_id \
+					in (' + ','.join([str(x) for x in ids]) + ')')
+		for res in cr.fetchall():
+			result[res[1]] = res[0] == 'sign_in' and 'present' or 'absent'
+		return result
+
 	_columns = {
 		'name' : fields.char("Employee", size=128, required=True),
 		'active' : fields.boolean('Active'),
 		'company_id': fields.many2one('res.company', 'Company'),
 		'address_id': fields.many2one('res.partner.address', 'Contact address'),
-		'state' : fields.selection([('absent', 'Absent'), ('present', 'Present')], 'Attendance', readonly=True),
+		'state': fields.function(_state, method=True, type='selection', selection=[('absent', 'Absent'), ('present', 'Present')], string='Attendance'),
 		'started' : fields.date("Started on"),
 		'notes': fields.text('Notes'),
 		'attendances' : fields.one2many('hr.attendance', 'employee_id', "Employee's attendances"),
@@ -111,7 +132,6 @@ class hr_employee(osv.osv):
 		'holiday_max' : fields.integer("Number of holidays"),
 		'parent_id': fields.many2one('hr.employee', 'Boss', select=True),
 		'child_ids': fields.one2many('hr.employee', 'parent_id','Subordinates'),
-		'bank_account': fields.char("Bank Account", size=16),
 	}
 	_defaults = {
 		'active' : lambda *a: True,
@@ -124,19 +144,20 @@ class hr_employee(osv.osv):
 			res = {'action':'action', 'employee_id':emp.id}
 			if dt:
 				res['name'] = dt
-			att_id = self.pool.get('hr.attendance').create(cr, uid, res)
+			att_id = self.pool.get('hr.attendance').create(cr, uid, res, context=context)
 		return True
 
 	def sign_out(self, cr, uid, ids, context={}, dt=False, *args):
+		id = False
 		for emp in self.browse(cr, uid, ids):
 			if not self._action_check(cr, uid, emp.id, dt, context):
 				raise osv.except_osv('Warning', 'You tried to sign out with a date anterior to another event !\nTry to contact the administrator to correct attendances.')
 			res = {'action':'sign_out', 'employee_id':emp.id}
 			if dt:
 				res['name'] = dt
-			att_id = self.pool.get('hr.attendance').create(cr, uid, res)
-			self.write(cr, uid, [emp.id], {'state':'absent'})
-		return True
+			att_id = self.pool.get('hr.attendance').create(cr, uid, res, context=context)
+			id = att_id
+		return id
 
 	def _action_check(self, cr, uid, emp_id, dt=False,context={}):
 		cr.execute('select max(name) from hr_attendance where employee_id=%d', (emp_id,))
@@ -144,15 +165,15 @@ class hr_employee(osv.osv):
 		return not (res and (res[0]>=(dt or time.strftime('%Y-%m-%d %H:%M:%S'))))
 
 	def sign_in(self, cr, uid, ids, context={}, dt=False, *args):
+		id = False
 		for emp in self.browse(cr, uid, ids):
 			if not self._action_check(cr, uid, emp.id, dt, context):
 				raise osv.except_osv('Warning', 'You tried to sign in with a date anterior to another event !\nTry to contact the administrator to correct attendances.')
 			res = {'action':'sign_in', 'employee_id':emp.id}
 			if dt:
 				res['name'] = dt
-			self.pool.get('hr.attendance').create(cr, uid, res)
-			self.write(cr, uid, [emp.id], {'state':'present'})
-		return True
+			id = self.pool.get('hr.attendance').create(cr, uid, res, context=context)
+		return id
 
 hr_employee()
 
@@ -163,8 +184,8 @@ class hr_timesheet(osv.osv):
 		'name' : fields.char("Name", size=64, required=True),
 		'dayofweek': fields.selection([('0','Monday'),('1','Tuesday'),('2','Wednesday'),('3','Thursday'),('4','Friday'),('5','Saturday'),('6','Sunday')], 'Day of week'),
 		'date_from' : fields.date('Starting date'),
-		'hour_from' : fields.char('Work from', size=8),
-		'hour_to' : fields.char("Work to", size=8),
+		'hour_from' : fields.char('Work from', size=8, required=True),
+		'hour_to' : fields.char("Work to", size=8, required=True),
 		'tgroup_id' : fields.many2one("hr.timesheet.group", "Employee's timesheet group", select=True),
 	}
 	_order = 'dayofweek, hour_from'
@@ -192,10 +213,10 @@ class hr_attendance(osv.osv):
 	_name = "hr.attendance"
 	_description = "Attendance"
 	_columns = {
-		'name' : fields.datetime('Date'),
-		'action' : fields.selection([('sign_in', 'Sign In'), ('sign_out', 'Sign Out'),('action','Action')], 'Action'),
+		'name' : fields.datetime('Date', required=True),
+		'action' : fields.selection([('sign_in', 'Sign In'), ('sign_out', 'Sign Out'),('action','Action')], 'Action', required=True),
 		'action_desc' : fields.many2one("hr.action.reason", "Action reason", domain="[('action_type', '=', action)]"),
-		'employee_id' : fields.many2one('hr.employee', 'Employee',required=True, select=True),
+		'employee_id' : fields.many2one('hr.employee', 'Employee', required=True, select=True),
 	}
 	_defaults = {
 		'name' : lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
