@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: hr_timesheet.py 4960 2006-12-21 09:25:34Z ced $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -45,7 +45,9 @@ hr_employee()
 class hr_analytic_timesheet(osv.osv):
 	_name = "hr.analytic.timesheet"
 	_table = 'hr_analytic_timesheet'
+	_description = "Timesheet line"
 	_inherits = {'account.analytic.line': 'line_id'}
+	_order = "id desc"
 	_columns = {
 		'line_id' : fields.many2one('account.analytic.line', 'Analytic line', ondelete='cascade'),
 	}
@@ -57,12 +59,16 @@ class hr_analytic_timesheet(osv.osv):
 		self.pool.get('account.analytic.line').unlink(cr, uid, toremove.keys(), context)
 		return super(hr_analytic_timesheet, self).unlink(cr, uid, ids, context)
 
+
 	def on_change_unit_amount(self, cr, uid, id, prod_id, unit_amount, unit, context={}):
-		return self.pool.get('account.analytic.line').on_change_unit_amount(cr, uid, id, prod_id, unit_amount,unit, context)
+		res = {}
+		if prod_id and unit_amount:
+			res = self.pool.get('account.analytic.line').on_change_unit_amount(cr, uid, id, prod_id, unit_amount,unit, context)
+		return res
 
 	def _getEmployeeProduct(self, cr, uid, context):
 		emp_obj = self.pool.get('hr.employee')
-		emp_id = emp_obj.search(cr, uid, [('user_id', '=', uid)])
+		emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
 		if emp_id:
 			emp=emp_obj.browse(cr, uid, emp_id[0], context)
 			if emp.product_id:
@@ -71,7 +77,7 @@ class hr_analytic_timesheet(osv.osv):
 
 	def _getEmployeeUnit(self, cr, uid, context):
 		emp_obj = self.pool.get('hr.employee')
-		emp_id = emp_obj.search(cr, uid, [('user_id', '=', uid)])
+		emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
 		if emp_id:
 			emp=emp_obj.browse(cr, uid, emp_id[0], context)
 			if emp.product_id:
@@ -80,19 +86,20 @@ class hr_analytic_timesheet(osv.osv):
 
 	def _getGeneralAccount(self, cr, uid, context):
 		emp_obj = self.pool.get('hr.employee')
-		emp_id = emp_obj.search(cr, uid, [('user_id', '=', uid)])
+		emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
 		if emp_id:
 			emp = self.pool.get('hr.employee').browse(cr, uid, emp_id[0], context=context)
 			if bool(emp.product_id):
-				a =  emp.product_id.product_tmpl_id.property_account_income
+				a =  emp.product_id.product_tmpl_id.property_account_expense.id
 				if not a:
-					a = emp.product_id.categ_id.property_account_income_categ
-				return a[0]
+					a = emp.product_id.categ_id.property_account_expense_categ.id
+				if a:
+					return a
 		return False
 
 	def _getAnalyticJournal(self, cr, uid, context):
 		emp_obj = self.pool.get('hr.employee')
-		emp_id = emp_obj.search(cr, uid, [('user_id', '=', uid)])
+		emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
 		if emp_id:
 			emp = self.pool.get('hr.employee').browse(cr, uid, emp_id[0], context=context)
 			if emp.journal_id:
@@ -105,14 +112,21 @@ class hr_analytic_timesheet(osv.osv):
 		'product_id' : _getEmployeeProduct,
 		'general_account_id' : _getGeneralAccount,
 		'journal_id' : _getAnalyticJournal,
-		'user_id' : lambda obj, cr, uid, ctx : uid,
+		'date' : lambda self,cr,uid,ctx: ctx.get('date', time.strftime('%Y-%m-%d')),
+		'user_id' : lambda obj, cr, uid, ctx : ctx.get('user_id', uid),
 	}
 
-	def on_change_account_id(self, cr, uid, ids, account_id):
-		return {}
-		if not account_id:
+
+	def on_change_user_id(self, cr, uid, ids, user_id):
+		if not user_id:
 			return {}
-		account = self.pool.get('account.analytic.account').read(cr, uid, [account_id], ['name'])[0]['name']
-		user = self.pool.get('res.users').read(cr, uid, [uid], ['name'])[0]['name']
-		return {'value' : {'name' : '%s (%s)' % (account, user)}}
+		return {'value' : {
+			'product_id' : self._getEmployeeProduct(cr,user_id, context= {}),
+			'product_uom_id' : self._getEmployeeUnit(cr, user_id, context= {}),
+			'general_account_id' :self. _getGeneralAccount(cr, user_id, context= {}),
+			'journal_id' : self._getAnalyticJournal(cr, user_id, context= {}),
+						   }}
+
+
+
 hr_analytic_timesheet()
