@@ -29,3 +29,5 @@
 
 import hr_timesheet_invoice_create
 import account_analytic_profit
+import hr_timesheet_final_invoice_create
+import wizard_account_analytic_cost_ledger_report
