@@ -247,7 +247,8 @@ class mrp_bom(osv.osv):
 			if bom.routing_id:
 				for wc_use in bom.routing_id.workcenter_lines:
 					wc = wc_use.workcenter_id
-					cycle = factor * wc_use.cycle_nbr
+					d, m = divmod(factor, wc_use.workcenter_id.capacity_per_cycle)
+					cycle = (d + (m and 1.0 or 0.0)) * wc_use.cycle_nbr
 					result2.append({
 						'name': bom.routing_id.name,
 						'workcenter_id': wc.id,
@@ -318,21 +319,21 @@ class mrp_production(osv.osv):
 		'location_src_id': fields.many2one('stock.location', 'Production Location', required=True),
 		'location_dest_id': fields.many2one('stock.location', 'Destination Location', required=True),
 
-		'date_planned': fields.date('Planned date', required=True),
+		'date_planned': fields.date('Scheduled date', required=True),
 		'date_start': fields.datetime('Start Date'),
 		'date_finnished': fields.datetime('End Date'),
 
 		'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[('bom_id','=',False)]),
 
-		'picking_id': fields.many2one('stock.picking', 'Picking list'),
-		'move_prod_id': fields.many2one('stock.move', 'Move Product'),
+		'picking_id': fields.many2one('stock.picking', 'Packing list', readonly=True),
+		'move_prod_id': fields.many2one('stock.move', 'Move product', readonly=True),
 		'move_lines': fields.many2many('stock.move', 'mrp_production_move_ids', 'production_id', 'move_id', 'Products Consummed'),
 
 		'move_created_ids': fields.one2many('stock.move', 'production_id', 'Moves Created'),
-		'product_lines': fields.one2many('mrp.production.product.line', 'production_id', 'Material Planned'),
+		'product_lines': fields.one2many('mrp.production.product.line', 'production_id', 'Scheduled goods'),
 		'workcenter_lines': fields.one2many('mrp.production.workcenter.line', 'production_id', 'Workcenters Utilisation'),
 
-		'state': fields.selection([('draft','Draft'),('picking_except', 'Picking Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Canceled'),('done','Done')],'State', readonly=True)
+		'state': fields.selection([('draft','Draft'),('picking_except', 'Packing Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Canceled'),('done','Done')],'State', readonly=True)
 	}
 	_defaults = {
 		'priority': lambda *a: '1',
@@ -340,7 +341,8 @@ class mrp_production(osv.osv):
 		'product_qty':  lambda *a: 1.0,
 		'name': lambda x,y,z,c: x.pool.get('ir.sequence').get(y,z,'mrp.production') or '/',
 	}
-	_order = 'date_planned desc, priority desc';
+	_order = 'date_planned asc, priority desc';
+
 	def product_id_change(self, cr, uid, ids, product):
 		if not product:
 			return {}
@@ -391,32 +393,31 @@ class mrp_production(osv.osv):
 		self.write(cr, uid, ids, {'state':'cancel','move_lines':[(6,0,[])]})
 		return True
 
-	#
-	# BUGFIX: may be a bug here; lot_lines are unreserved for a few seconds;
-	#         between the end of the picking list and the call to this function
-	#
+	#XXX: may be a bug here; lot_lines are unreserved for a few seconds;
+	#     between the end of the picking list and the call to this function
 	def action_ready(self, cr, uid, ids):
 		self.write(cr, uid, ids, {'state':'ready'})
 		return True
 
-	#
-	# Review materials in function in_prod and prod_end.
-	#
+	#TODO Review materials in function in_prod and prod_end.
 	def action_production_end(self, cr, uid, ids):
 		move_ids = []
 		for production in self.browse(cr, uid, ids):
 			for res in production.move_lines:
 				for move in production.move_created_ids:
-					cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%d,%d)', (res.id, move.id))
+					#XXX must use the orm
+					cr.execute('INSERT INTO stock_move_history_ids \
+							(parent_id, child_id) VALUES (%d,%d)',
+							(res.id, move.id))
 				move_ids.append(res.id)
 			if production.move_created_ids:
-				# There we should handle the residus move creation
+				#TODO There we should handle the residus move creation
 				vals= {'state':'confirmed'}
 				new_moves = [x.id for x in production.move_created_ids]
 				self.pool.get('stock.move').write(cr, uid, new_moves, vals)
 			else:
-				# Why is it there ? Aren't we suppose to already have a created_move ?
-				source = production.product_id.product_tmpl_id.property_stock_production[0]
+				#XXX Why is it there ? Aren't we suppose to already have a created_move ?
+				source = production.product_id.product_tmpl_id.property_stock_production.id
 				vals = {
 					'name':'PROD:'+production.name,
 					'date_planned': production.date_planned,
@@ -429,12 +430,16 @@ class mrp_production(osv.osv):
 					'state': 'confirmed'
 				}
 				new_moves = [self.pool.get('stock.move').create(cr, uid, vals)]
-				self.write(cr, uid, [production.id], {'move_created_ids': [(6, 'WTF', new_moves)]})
+				self.write(cr, uid, [production.id],
+						{'move_created_ids': [(6, 'WTF', new_moves)]})
+			if not production.date_finnished:
+				self.write(cr, uid, [production.id],
+						{'date_finnished': time.strftime('%Y-%m-%d %H:%M:%S')})
 			self.pool.get('stock.move').check_assign(cr, uid, new_moves)
 			self.pool.get('stock.move').action_done(cr, uid, new_moves)
 			self._costs_generate(cr, uid, production)
 		self.pool.get('stock.move').action_done(cr, uid, move_ids)
-		self.write(cr,  uid, ids, {'state':'done'})
+		self.write(cr,  uid, ids, {'state': 'done'})
 		return True
 
 	def _costs_generate(self, cr, uid, production):
@@ -470,12 +475,15 @@ class mrp_production(osv.osv):
 		return amount
 
 	def action_in_production(self, cr, uid, ids):
+		move_ids = []
 		for production in self.browse(cr, uid, ids):
-			ids2 = []
 			for res in production.move_lines:
-				ids2.append(res.id)
-			self.pool.get('stock.move').action_done(cr, uid, ids2)
-		self.write(cr, uid, ids, {'state':'in_production'})
+				move_ids.append(res.id)
+			if not production.date_start:
+				self.write(cr, uid, [production.id],
+						{'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
+		self.pool.get('stock.move').action_done(cr, uid, move_ids)
+		self.write(cr, uid, ids, {'state': 'in_production'})
 		return True
 
 	def test_if_product(self, cr, uid, ids):
@@ -493,7 +501,7 @@ class mrp_production(osv.osv):
 				self.action_compute(cr, uid, [production.id])
 				production = self.browse(cr, uid, [production.id])[0]
 			picking_id = self.pool.get('stock.picking').create(cr, uid, {
-				'origin': (production.origin or '') +':'+production.name,
+				'origin': (production.origin or '').split(':')[0] +':'+production.name,
 				'type': 'internal',
 				'move_type': 'one',
 				'state': 'auto',
@@ -504,7 +512,7 @@ class mrp_production(osv.osv):
 			if production.move_prod_id:
 				self.pool.get('stock.move').write(cr, uid, [production.move_prod_id.id], {'location_id':production.location_dest_id.id})
 
-			source = production.product_id.product_tmpl_id.property_stock_production[0]
+			source = production.product_id.product_tmpl_id.property_stock_production.id
 			data = {
 				'name':'PROD:'+production.name,
 				'date_planned': production.date_planned,
@@ -549,8 +557,8 @@ class mrp_production(osv.osv):
 						'state': 'waiting',
 					})
 				proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
-					'name': production.name+':'+ (production.origin or ''),
-					'origin': production.name+':'+ (production.origin or ''),
+					'name': (production.origin or '').split(':')[0] + ':' + production.name,
+					'origin': (production.origin or '').split(':')[0] + ':' + production.name,
 					'date_planned': newdate,
 					'product_id': line.product_id.id,
 					'product_qty': line.product_qty,
@@ -574,6 +582,7 @@ class mrp_production(osv.osv):
 
 mrp_production()
 
+
 class stock_move(osv.osv):
 	_name = 'stock.move'
 	_inherit = 'stock.move'
@@ -581,6 +590,7 @@ class stock_move(osv.osv):
 	_columns = {
 			'production_id': fields.many2one('mrp.production', 'Production', select=True),
 			}
+
 stock_move()
 
 class mrp_production_workcenter_line(osv.osv):
@@ -603,7 +613,7 @@ mrp_production_workcenter_line()
 
 class mrp_production_product_line(osv.osv):
 	_name = 'mrp.production.product.line'
-	_description = 'Production planned products'
+	_description = 'Production scheduled products'
 	_columns = {
 		'name': fields.char('Name', size=64, required=True),
 		'product_id': fields.many2one('product.product', 'Product', required=True),
@@ -627,7 +637,7 @@ class mrp_procurement(osv.osv):
 		'name': fields.char('Name', size=64, required=True),
 		'origin': fields.char('Origin', size=64),
 		'priority': fields.selection([('0','Not urgent'),('1','Normal'),('2','Urgent'),('3','Very Urgent')], 'Priority', required=True),
-		'date_planned': fields.date('Date Promised', required=True),
+		'date_planned': fields.date('Scheduled date', required=True),
 		'date_close': fields.date('Date Closed'),
 		'product_id': fields.many2one('product.product', 'Product', required=True),
 		'product_qty': fields.float('Quantity', required=True),
@@ -660,8 +670,9 @@ class mrp_procurement(osv.osv):
 	def check_move_cancel(self, cr, uid, ids, context={}):
 		res = True
 		for procurement in self.browse(cr, uid, ids, context):
-			if not procurement.move_id.state=='cancel':
-				res = False
+			if procurement.move_id:
+				if not procurement.move_id.state=='cancel':
+					res = False
 		return res
 
 	def check_move_done(self, cr, uid, ids, context={}):
@@ -751,7 +762,7 @@ class mrp_procurement(osv.osv):
 				if not procurement.move_id:
 					source = procurement.location_id.id
 					if procurement.procure_method=='make_to_order':
-						source = procurement.product_id.product_tmpl_id.property_stock_procurement[0]
+						source = procurement.product_id.product_tmpl_id.property_stock_procurement.id
 					id = self.pool.get('stock.move').create(cr, uid, {
 						'name': 'PROC:'+procurement.name,
 						'location_id': source,
@@ -771,7 +782,7 @@ class mrp_procurement(osv.osv):
 		return True
 
 	def action_move_assigned(self, cr, uid, ids):
-		self.write(cr, uid, ids, {'state':'running','message':'MTS: products assigned.'})
+		self.write(cr, uid, ids, {'state':'running','message':'from stock: products assigned.'})
 		return True
 
 	def _check_make_to_stock_service(self, cr, uid, procurement, context={}):
@@ -785,7 +796,7 @@ class mrp_procurement(osv.osv):
 				ok = ok and self.pool.get('stock.move').action_assign(cr, uid, [id])
 				cr.execute('select count(id) from stock_warehouse_orderpoint where product_id=%d', (procurement.product_id.id,))
 				if not cr.fetchone()[0]:
-					cr.execute('update mrp_procurement set message=%s where id=%d', ('MTS and no orderpoint defined', procurement.id))
+					cr.execute('update mrp_procurement set message=%s where id=%d', ('from stock and no minimum orderpoint rule defined', procurement.id))
 		return ok
 
 	def action_produce_assign_service(self, cr, uid, ids, context={}):
@@ -800,7 +811,6 @@ class mrp_procurement(osv.osv):
 			loc_id = procurement.location_id.id
 			newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.produce_delay or 0.0)
 			produce_id = self.pool.get('mrp.production').create(cr, uid, {
-				#'name': procurement.name,
 				'origin': procurement.origin,
 				'product_id': procurement.product_id.id,
 				'product_qty': procurement.product_qty,
@@ -811,7 +821,8 @@ class mrp_procurement(osv.osv):
 				'move_prod_id': res_id,
 			})
 			self.write(cr, uid, [procurement.id], {'state':'running'})
-			bom_result = self.pool.get('mrp.production').action_compute(cr, uid, [produce_id], properties=procurement.property_ids)
+			bom_result = self.pool.get('mrp.production').action_compute(cr, uid,
+					[produce_id], properties=[x.id for x in procurement.property_ids])
 			wf_service = netsvc.LocalService("workflow")
 			wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)
 		return produce_id
@@ -823,7 +834,7 @@ class mrp_procurement(osv.osv):
 			partner = procurement.product_id.seller_ids[0].name
 			partner_id = partner.id
 			address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
-			pricelist_id = partner.property_product_pricelist_purchase[0]
+			pricelist_id = partner.property_product_pricelist_purchase.id
 
 			uom_id = procurement.product_id.uom_po_id.id
 
@@ -893,18 +904,58 @@ class mrp_procurement(osv.osv):
 		for id in ids:
 			wf_service.trg_trigger(uid, 'mrp.procurement', id, cr)
 		return res
+	def run_scheduler(self, cr, uid, user_id=False, schedule_cycle=1.0,\
+			po_cycle=1.0, po_lead=1.0, security_lead=50.0, picking_lead=1.0,\
+			automatic=False, use_new_cursor=False, context=None):
+		'''
+		use_new_cursor: False or the dbname
+		'''
+		if not context:
+			context={}
+		self.run_procure_confirm(cr, uid, schedule_cycle=schedule_cycle,\
+				po_cycle=po_cycle, po_lead=po_lead, security_lead=security_lead,\
+				picking_lead=picking_lead, user_id=user_id,\
+				use_new_cursor=use_new_cursor, context=context)
+		self.run_orderpoint_confirm(cr, uid, automatic=automatic,\
+				use_new_cursor=use_new_cursor, context=context, user_id=user_id)
+
+	def run_procure_confirm(self, cr, uid, user_id=False, schedule_cycle=1.0,\
+			po_cycle=1.0, po_lead=1.0, security_lead=50.0, picking_lead=1.0, \
+			use_new_cursor=False, context=None):
+		'''
+		use_new_cursor: False or the dbname
+		'''
+		from wizard.schedulers import _procure_confirm
+		if not context:
+			context={}
+		_procure_confirm(self, cr, uid, schedule_cycle=schedule_cycle,\
+				po_cycle=po_cycle, po_lead=po_lead, security_lead=security_lead,\
+				picking_lead=picking_lead, user_id=user_id, use_new_cursor=use_new_cursor,\
+				context=context)
+
+	def run_orderpoint_confirm(self, cr, uid, automatic=False, use_new_cursor=False,\
+			context=None, user_id=False):
+		'''
+		use_new_cursor: False or the dbname
+		'''
+		from wizard.schedulers import _procure_orderpoint_confirm
+		if not context:
+			context={}
+		_procure_orderpoint_confirm(self, cr, uid, automatic=automatic,\
+				use_new_cursor=use_new_cursor, context=context, user_id=user_id)
+
 mrp_procurement()
 
 #CHECKME: why the **** is this object defined here and not in stock???
 class stock_warehouse_orderpoint(osv.osv):
 	_name = "stock.warehouse.orderpoint"
-	_description = "Warehouse Orderpoint"
+	_description = "Orderpoint minimum rule"
 	_columns = {
 		'name': fields.char('Name', size=32, required=True),
 		'active': fields.boolean('Active'),
 		'logic': fields.selection([('max','Order to Max'),('price','Best price (not yet active!)')], 'Reordering Mode', required=True),
-		'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True, relate=True),
-		'product_id': fields.many2one('product.product', 'Product', required=True, domain=[('type','=','product')], relate=True),
+		'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True),
+		'product_id': fields.many2one('product.product', 'Product', required=True, domain=[('type','=','product')]),
 		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True ),
 		'product_min_qty': fields.float('Min Quantity', required=True),
 		'product_max_qty': fields.float('Max Quantity', required=True),
@@ -916,6 +967,7 @@ class stock_warehouse_orderpoint(osv.osv):
 		'logic': lambda *a: 'max',
 		'qty_multiple': lambda *a: 1,
 		'name': lambda x,y,z,c: x.pool.get('ir.sequence').get(y,z,'mrp.warehouse.orderpoint') or '',
+		'product_uom': lambda sel, cr, uid, context: context.get('product_uom', False),
 	}
 	def onchange_product_id(self, cr, uid, ids, product_id, context={}):
 		if product_id:
@@ -926,3 +978,28 @@ class stock_warehouse_orderpoint(osv.osv):
 stock_warehouse_orderpoint()
 
 
+class StockMove(osv.osv):
+	_inherit = 'stock.move'
+
+	_columns = {
+			'procurements': fields.one2many('mrp.procurement', 'move_id', 'Procurements'),
+			}
+
+StockMove()
+
+
+class StockPicking(osv.osv):
+	_inherit = 'stock.picking'
+
+	def test_finnished(self, cursor, user, ids):
+		wf_service = netsvc.LocalService("workflow")
+		res = super(StockPicking, self).test_finnished(cursor, user, ids)
+		for picking in self.browse(cursor, user, ids):
+			for move in picking.move_lines:
+				if move.state == 'done' and move.procurements:
+					for procurement in move.procurements:
+						wf_service.trg_validate(user, 'mrp.procurement',
+								procurement.id, 'button_check', cursor)
+		return res
+
+StockPicking()
