@@ -1,10 +1,10 @@
 {
 	"name" : "Project Management",
-	"version": "0.1",
+	"version": "1.0",
 	"author" : "Tiny",
 	"website" : "http://tinyerp.com/module_project.html",
 	"category" : "Generic Modules/Projects & Services",
-	"depends" : ["product", "account", 'mrp', 'sale'],
+	"depends" : ["product", "account", 'mrp', 'sale', 'base'],
 	"description": "Project management module that track multi-level projects, tasks, works done on tasks, eso. It is able to render planning, order tasks, eso.",
 	"init_xml" : [],
 	"demo_xml" : ["project_demo.xml"],
