@@ -26,8 +26,9 @@
 #
 ##############################################################################
 
-import tools
 import purchase
 import partner
+import stock
 import wizard
 import report
+import stock
