@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: wizard_group.py 5190 2007-01-10 06:30:51Z pinky $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -59,7 +59,7 @@ def _mergeOrders(self, cr, uid, data, context):
 		list_key = []
 		for field in fields:
 			field_val = getattr(br, field)
-			if field in ('product_id','move_dest_id'):
+			if field in ('product_id','move_dest_id', 'account_analytic_id'):
 				if not field_val:
 					field_val = False
 			if isinstance(field_val, browse_record):
@@ -86,18 +86,18 @@ def _mergeOrders(self, cr, uid, data, context):
 				'dest_address_id': porder.dest_address_id.id,
 				'warehouse_id': porder.warehouse_id.id,
 				'location_id': porder.location_id.id,
-				'project_id': porder.project_id.id,
 				'pricelist_id': porder.pricelist_id.id,
 				'state': 'draft',
 				'order_line': {},
-				'notes': '%s' % (porder.notes,),
+				'notes': '%s' % (porder.notes or '',),
 			})
 		else:
 			#order_infos['name'] += ', %s' % porder.name
-			order_infos['notes'] += ('\n%s' % (porder.notes,))
+			if porder.notes:
+				order_infos['notes'] += ('\n%s' % (porder.notes,))
 
 		for order_line in porder.order_line:
-			line_key = make_key(order_line, ('name', 'date_planned', 'taxes_id', 'price_unit', 'notes', 'product_id', 'move_dest_id'))	
+			line_key = make_key(order_line, ('name', 'date_planned', 'taxes_id', 'price_unit', 'notes', 'product_id', 'move_dest_id', 'account_analytic_id'))
 			o_line = order_infos['order_line'].setdefault(line_key, {})
 			if o_line:
 				# merge the line with an existing line
@@ -137,7 +137,7 @@ def _mergeOrders(self, cr, uid, data, context):
 
 	return {
 		'domain': "[('id','in', ["+','.join(map(str,allorders))+"])]",
-		'name': 'Purchase Orders',
+		'name': 'Purchase orders',
 		'view_type': 'form',
 		'view_mode': 'tree,form',
 		'res_model': 'purchase.order',
