@@ -28,7 +28,7 @@
 ##############################################################################
 
 import StringIO
-import sql_db
+import pooler
 
 from report.render import render
 from report.interface import report_int
@@ -50,13 +50,12 @@ class external_pdf(render):
 		return self.pdf
 
 class report_tasks(report_int):
-	def create(self, uid, ids, datas, context={}):
+	def create(self, cr, uid, ids, datas, context={}):
 		io = StringIO.StringIO()
 
 		canv = canvas.init(fname=io, format='pdf')
 		canv.set_author("Tiny ERP")
 
-		cr = sql_db.db.cursor()
 		cr.execute('select id,date_start,date_stop from scrum_sprint where id=%d', (datas['id'],))
 		for (id,date_start,date_stop) in cr.fetchall():
 			date_to_int = lambda x: int(x.ticks())
