@@ -1,8 +1,8 @@
 ##############################################################################
 #
-# Copyright (c) 2005 TINY SPRL. (http://tiny.be) All Rights Reserved.
+# Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: ups.py 2572 2006-03-29 14:41:30Z pinky $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -27,15 +27,9 @@
 #
 ##############################################################################
 
-import time, os
-import netsvc, sql_db
 from report.interface import report_rml
-from osv.osv import osv_pools
 
 class report_custom(report_rml):
-	def __init__(self, name, table, tmpl, xsl):
-		report_rml.__init__(self, name, table, tmpl, xsl)
-
 	def create(self, uid, ids, datas, context):
 		datas.setdefault('form', {})
 		datas['form'].setdefault('weight', 3.0)
