@@ -32,6 +32,7 @@ import time
 import netsvc
 from osv import fields,osv
 import ir
+from tools import config
 
 
 #----------------------------------------------------------
@@ -71,20 +72,21 @@ stock_lot()
 class stock_location(osv.osv):
 	_name = "stock.location"
 	_description = "Location"
+	_parent_name = "location_id"
 	_columns = {
 		'name': fields.char('Location Name', size=64, required=True),
 		'active': fields.boolean('Active'),
 		'usage': fields.selection([('supplier','Supplier Location'),('internal','Internal Location'),('customer','Customer Location'),('inventory','Inventory'),('procurement','Procurement'),('production','Production')], 'Location type'),
 		'allocation_method': fields.selection([('fifo','FIFO'),('lifo','LIFO'),('nearest','Nearest')], 'Allocation Method', required=True),
 
-		'account_id': fields.many2one('account.account', string='Inventory Account', domain=[('type','=','stock_inventory')]),
+		'account_id': fields.many2one('account.account', string='Inventory Account', domain=[('type','!=','view')]),
 		'location_id': fields.many2one('stock.location', 'Parent Location', select=True),
 		'child_ids': fields.one2many('stock.location', 'location_id', 'Contains'),
 
 		'comment': fields.text('Additional Information'),
-		'posx': fields.integer('Position X', required=True),
-		'posy': fields.integer('Position Y', required=True),
-		'posz': fields.integer('Position Z', required=True)
+		'posx': fields.integer('Corridor (X)', required=True),
+		'posy': fields.integer('Shelves (Y)', required=True),
+		'posz': fields.integer('Height (Z)', required=True)
 	}
 	_defaults = {
 		'active': lambda *a: 1,
@@ -95,28 +97,49 @@ class stock_location(osv.osv):
 		'posz': lambda *a: 0,
 	}
 
-	def _product_get_all_report(self, cr, uid, ids, product_ids=False, context={}):
-		return self._product_get_report(cr, uid, ids, product_ids, context, recursive=True)
-		
-	def _product_get_report(self, cr, uid, ids, product_ids=False, context={}, recursive=False):
+	def _product_get_all_report(self, cr, uid, ids, product_ids=False,
+			context=None):
+		return self._product_get_report(cr, uid, ids, product_ids, context,
+				recursive=True)
+
+	def _product_get_report(self, cr, uid, ids, product_ids=False,
+			context=None, recursive=False):
+		if context is None:
+			context = {}
+		product_obj = self.pool.get('product.product')
 		if not product_ids:
-			product_ids = self.pool.get('product.product').search(cr, uid, [])
+			product_ids = product_obj.search(cr, uid, [])
+
+		products = product_obj.browse(cr, uid, product_ids, context=context)
+		products_by_uom = {}
+		products_by_id = {}
+		for product in products:
+			products_by_uom.setdefault(product.uom_id.id, [])
+			products_by_uom[product.uom_id.id].append(product)
+			products_by_id.setdefault(product.id, [])
+			products_by_id[product.id] = product
+
 		result = []
 		for id in ids:
-			for prod_id in product_ids:
-				product = self.pool.get('product.product').browse(cr, uid, [prod_id])[0]
+			for uom_id in products_by_uom.keys():
 				fnc = self._product_get
 				if recursive:
 					fnc = self._product_all_get
-				qty = fnc(cr, uid, id, [prod_id], {'uom': product.uom_id.id})
-				if qty[prod_id]:
+				ctx = context.copy()
+				ctx['uom'] = uom_id
+				qty = fnc(cr, uid, id, [x.id for x in products_by_uom[uom_id]],
+						context=ctx)
+				for product_id in qty.keys():
+					if not qty[product_id]:
+						continue
+					product = products_by_id[product_id]
 					result.append({
 						'price': product.list_price,
 						'name': product.name,
-						'code': product.default_code, # used by lot_overview_all report !
+						'code': product.default_code, # used by lot_overview_all report!
 						'variants': product.variants or '',
 						'uom': product.uom_id.name,
-						'amount':qty[prod_id]
+						'amount': qty[product_id],
 					})
 		return result
 
@@ -133,6 +156,7 @@ class stock_location(osv.osv):
 		prod_ids_str = ','.join(map(str, product_ids))
 		location_ids_str = ','.join(map(str, ids))
 		results = []
+		results2 = []
 		if 'in' in what:
 			# all moves from a location out of the set to a location in the set
 			cr.execute(
@@ -144,11 +168,11 @@ class stock_location(osv.osv):
 				'and state in ('+states_str+') '\
 				'group by product_id,product_uom'
 			)
-			results += cr.fetchall()
+			results = cr.fetchall()
 		if 'out' in what:
 			# all moves from a location in the set to a location out of the set
 			cr.execute(
-				'select -sum(product_qty), product_id, product_uom '\
+				'select sum(product_qty), product_id, product_uom '\
 				'from stock_move '\
 				'where location_id in ('+location_ids_str+') '\
 				'and location_dest_id not in ('+location_ids_str+') '\
@@ -156,13 +180,18 @@ class stock_location(osv.osv):
 				'and state in ('+states_str+') '\
 				'group by product_id,product_uom'
 			)
-			results += cr.fetchall()
+			results2 = cr.fetchall()
 		uom_obj = self.pool.get('product.uom')
 		for amount, prod_id, prod_uom in results:
-			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount, context.get('uom', False))
+			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount,
+					context.get('uom', False))
 			res[prod_id] += amount
+		for amount, prod_id, prod_uom in results2:
+			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount,
+					context.get('uom', False))
+			res[prod_id] -= amount
 		return res
-		
+
 	def _product_get(self, cr, uid, id, product_ids=False, context={}, states=['done']):
 		ids = id and [id] or []
 		return self._product_get_multi_location(cr, uid, ids, product_ids, context, states)
@@ -229,8 +258,8 @@ class stock_move_lot(osv.osv):
 		'active': fields.boolean('Active'),
 		'state': fields.selection( (('draft','Draft'),('done','Moved')), 'State', readonly=True),
 		'serial': fields.char('Tracking Number', size=32),
-		'date_planned': fields.date('Planned Date'),
-		'date_moved': fields.date('Actual Date'),
+		'date_planned': fields.date('Scheduled date'),
+		'date_moved': fields.date('Actual date'),
 		'lot_id': fields.many2one('stock.lot','Lot', required=True),
 		'loc_dest_id': fields.many2one('stock.location', 'Destination Location', required=True),
 		'address_id': fields.many2one('res.partner.address', 'Destination Address'),
@@ -284,15 +313,18 @@ class stock_tracking(osv.osv):
 	}
 	_defaults = {
 		'active': lambda *a: 1,
-#		'name': lambda x,y,z,c: x.pool.get('ir.sequence').get(y,z,'stock.lot.tracking'),
 		'name' : make_sscc,
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 	}
-	#_sql = 'ALTER TABLE stock_tracking ADD CONSTRAINT stock_lot_tracking_unique UNIQUE (name,serial)'
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('serial','=',name)]+ args)
-		ids += self.search(cr, user, [('name',operator,name)]+ args)
-		return self.name_get(cr, user, ids)
+
+	def name_search(self, cr, user, name, args=None, operator='ilike', context=None, limit=80):
+		if not args:
+			args=[]
+		if not context:
+			context={}
+		ids = self.search(cr, user, [('serial','=',name)]+ args, limit=limit, context=context)
+		ids += self.search(cr, user, [('name',operator,name)]+ args, limit=limit, context=context)
+		return self.name_get(cr, user, ids, context)
 
 	def name_get(self, cr, uid, ids, context={}):
 		if not len(ids):
@@ -301,7 +333,7 @@ class stock_tracking(osv.osv):
 		return res
 
 	def unlink(self, cr ,uid, ids):
-		raise 'You can not remove a lot line !'
+		raise Exception, 'You can not remove a lot line !'
 stock_tracking()
 
 #----------------------------------------------------------
@@ -309,9 +341,9 @@ stock_tracking()
 #----------------------------------------------------------
 class stock_picking(osv.osv):
 	_name = "stock.picking"
-	_description = "Picking list"
+	_description = "Packing list"
 	_columns = {
-		'name': fields.char('Picking Name', size=64, required=True, select=True),
+		'name': fields.char('Packing name', size=64, required=True, select=True),
 		'origin': fields.char('Origin', size=64),
 		'type': fields.selection([('out','Sending Goods'),('in','Getting Goods'),('internal','Internal')], 'Shipping Type', required=True, select=True),
 		'active': fields.boolean('Active'),
@@ -322,30 +354,30 @@ class stock_picking(osv.osv):
 
 		'move_type': fields.selection([('direct','Direct Delivery'),('one','All at once')],'Delivery Method', required=True),
 		'state': fields.selection([
-			('draft','draft'),
-			('auto','waiting'),
-			('confirmed','confirmed'),
-			('assigned','assigned'),
-			('done','done'),
-			('cancel','cancel'),
+			('draft','Draft'),
+			('auto','Waiting'),
+			('confirmed','Confirmed'),
+			('assigned','Assigned'),
+			('done','Done'),
+			('cancel','Cancel'),
 			], 'State', readonly=True),
 		'date':fields.datetime('Date create'),
 
-		'move_lines': fields.one2many('stock.move', 'picking_id', 'Move Lines'),
+		'move_lines': fields.one2many('stock.move', 'picking_id', 'Move lines'),
 
-		'auto_picking': fields.boolean('Auto-Picking'),
+		'auto_picking': fields.boolean('Auto-Packing'),
 		'work': fields.boolean('Work todo'),
-		'loc_move_id': fields.many2one('stock.location', 'Move to Location'),
-		'address_id': fields.many2one('res.partner.address', 'Partner Address'),
-		'lot_id': fields.many2one('stock.lot', 'Consumer Lot Created'),
-		'move_lot_id': fields.many2one('stock.move.lot', 'Moves Created'),
+		'loc_move_id': fields.many2one('stock.location', 'Final location'),
+		'address_id': fields.many2one('res.partner.address', 'Partner'),
+		'lot_id': fields.many2one('stock.lot', 'Consumer lot created'),
+		'move_lot_id': fields.many2one('stock.move.lot', 'Moves created'),
 		'invoice_state':fields.selection([
 			("invoiced","Invoiced"),
 			("2binvoiced","To be invoiced"),
-			("none","No invoice")], "Invoice state"),
+			("none","Not from Packing")], "Invoice state"),
 	}
 	_defaults = {
-		'name': lambda *a: 'Unconfirmed picking',
+		'name': lambda *a: '/',
 		'work': lambda *a: 0,
 		'active': lambda *a: 1,
 		'state': lambda *a: 'draft',
@@ -387,16 +419,27 @@ class stock_picking(osv.osv):
 			wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
 		return True
 
+	def cancel_assign(self, cr, uid, ids, *args):
+		wf_service = netsvc.LocalService("workflow")
+		for pick in self.browse(cr, uid, ids):
+			move_ids = [x.id for x in pick.move_lines]
+			self.pool.get('stock.move').cancel_assign(cr, uid, move_ids)
+			wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
+		return True
+
 	def action_assign_wkf(self, cr, uid, ids):
 		self.write(cr, uid, ids, {'state':'assigned'})
 		return True
 
 	def test_finnished(self, cr, uid, ids):
-		for id in ids:
-			cr.execute('select id,state from stock_move where picking_id=%d', (id,))
-			for x in cr.fetchall():
-				if x[1] not in ('done','cancel'):
+		move_ids=self.pool.get('stock.move').search(cr,uid,[('picking_id','in',ids)])
+		
+		for move in self.pool.get('stock.move').browse(cr,uid,move_ids):
+			if move.state not in ('done','cancel') :
+				if move.product_qty != 0.0:
 					return False
+				else:
+					move.write(cr,uid,[move.id],{'state':'done'})
 		return True
 
 	def test_assigned(self, cr, uid, ids):
@@ -406,7 +449,7 @@ class stock_picking(osv.osv):
 			for move in pick.move_lines:
 				if (move.state in ('confirmed','draft')) and (mt=='one'):
 					return False
-				if (mt=='direct') and move.state=='assigned':
+				if (mt=='direct') and (move.state=='assigned') and (move.product_qty):
 					return True
 				ok = ok and (move.state in ('cancel','done','assigned'))
 		return ok
@@ -446,7 +489,7 @@ class stock_picking(osv.osv):
 			if len(todo):
 				self.pool.get('stock.move').action_done(cr, uid, todo)
 
-				lot_id = self.pool.get('stock.lot').create(cr, uid, {'name':'PICK:'+pick.name})
+				lot_id = self.pool.get('stock.lot').create(cr, uid, {'name':pick.name})
 				self.pool.get('stock.move').write(cr,uid, todo, {'lot_id':lot_id})
 				self.write(cr, uid, [pick.id], {'lot_id':lot_id})
 
@@ -461,101 +504,148 @@ class stock_picking(osv.osv):
 					self.write(cr, uid, [pick.id], {'move_lot_id':id})
 		return True
 
-	def action_invoice_create(self, cr, uid, ids, journal_id=False, group=False, type='out_invoice', context={}):
-		res={}
-		pgroup = {}
-		get_ids = lambda y: map(lambda x: x.id, y or [])
-		sales = {}
-		for p in self.browse(cr,uid,ids, context):
-			if p.invoice_state<>'2binvoiced':
+	def _get_address_invoice(self, cursor, user, picking):
+		'''Return {'contact': address, 'invoice': address} for invoice'''
+		partner_obj = self.pool.get('res.partner')
+		partner = picking.address_id.partner_id
+
+		return partner_obj.address_get(cursor, user, [partner.id],
+				['contact', 'invoice'])
+
+	def _get_comment_invoice(self, cursor, user, picking):
+		'''Return comment string for invoice'''
+		return picking.note or ''
+
+	def _get_price_unit_invoice(self, cursor, user, move_line, type):
+		'''Return the price unit for the move line'''
+		if type in ('in_invoice', 'in_refund'):
+			return move_line.product_id.standard_price
+		else:
+			return move_line.product_id.list_price
+	
+	def _get_discount_invoice(self, cursor, user, move_line):
+		'''Return the discount for the move line'''
+		return 0.0
+
+	def _get_taxes_invoice(self, cursor, user, move_line, type):
+		'''Return taxes ids for the move line'''
+		if type in ('in_invoice', 'in_refund'):
+			return [x.id for x in move_line.product_id.supplier_taxes_id]
+		else:
+			return [x.id for x in move_line.product_id.taxes_id]
+
+	def _get_account_analytic_invoice(self, cursor, user, picking, move_line):
+		return False
+
+	def _invoice_line_hook(self, cursor, user, move_line, invoice_line_id):
+		'''Call after the creation of the invoice line'''
+		return
+
+	def _invoice_hook(self, cursor, user, picking, invoice_id):
+		'''Call after the creation of the invoice'''
+		return
+
+	def action_invoice_create(self, cursor, user, ids, journal_id=False,
+			group=False, type='out_invoice', context=None):
+		'''Return ids of created invoices for the pickings'''
+		invoice_obj = self.pool.get('account.invoice')
+		invoice_line_obj = self.pool.get('account.invoice.line')
+		invoices_group = {}
+		res = {}
+
+		for picking in self.browse(cursor, user, ids, context=context):
+			if picking.invoice_state != '2binvoiced':
 				continue
-			a = p.address_id.partner_id.property_account_receivable[0]
-			if p.address_id.partner_id and p.address_id.partner_id.property_payment_term:
-				pay_term = p.address_id.partner_id.property_payment_term[0]
+
+			partner = picking.address_id.partner_id
+			if type in ('out_invoice', 'out_refund'):
+				account_id = partner.property_account_receivable.id
 			else:
-				pay_term = False
+				account_id = partner.property_account_payable.id
+			payment_term_id = False
+			if partner.property_payment_term:
+				payment_term_id = partner.property_payment_term.id
 
-			if p.sale_id:
-				pinv_id = p.sale_id.partner_invoice_id.id
-				pcon_id = p.sale_id.partner_order_id.id
-				if p.sale_id.id not in sales:
-					sales[p.sale_id.id] = [x.id for x in p.sale_id.invoice_ids]
+			address_contact_id, address_invoice_id = \
+					self._get_address_invoice(cursor, user, picking).values()
+
+			comment = self._get_comment_invoice(cursor, user, picking)
+
+			if group and partner.id in invoices_group:
+				invoice_id = invoices_group[partner.id]
 			else:
-				#
-				# ideal: get_address('invoice') on partner
-				#
-				pinv_id = p.address_id.id
-				pcon_id = p.address_id.id
-
-			val = {
-				'name': p.name,
-				'origin': p.name+':'+p.origin,
-				'type': type,
-				'reference': "P%dSO%d"%(p.address_id.partner_id.id,p.id),
-				'account_id': a,
-				'partner_id': p.address_id.partner_id.id,
-				'address_invoice_id': pinv_id,
-				'address_contact_id': pcon_id,
-				'project_id': (p.sale_id and p.sale_id.project_id.id) or False,
-				'comment': (p.note or '') + '\n' + (p.sale_id and p.sale_id.note or ''),
-				'payment_term': pay_term,
-			}
-			if p.sale_id:
-				val['currency_id'] = (p.sale_id and p.sale_id.pricelist_id.currency_id.id) or False
-			if journal_id:
-				val['journal_id'] = journal_id
-
-			if group and p.address_id.partner_id.id in pgroup:
-				invoice_id= pgroup[p.address_id.partner_id.id]
-			else:
-				invoice_id = self.pool.get('account.invoice').create(cr, uid, val ,context= context)
-				pgroup[p.address_id.partner_id.id] = invoice_id
-
-
-			res[p.id]= invoice_id
-
-			for line in p.move_lines:
-				if line.sale_line_id:
-					tax_ids = map(lambda x: x.id, line.sale_line_id.tax_id)
-				else:
-					tax_ids = map(lambda x: x.id, line.product_id.taxes_id)
-				a =  line.product_id.product_tmpl_id.property_account_income
-				if not a:
-					a = line.product_id.categ_id.property_account_income_categ
-				account_id =  a[0]
-				punit = line.sale_line_id and line.sale_line_id.price_unit or line.product_id.list_price
-				if type in ('in_invoice','in_refund'):
-					punit = line.product_id.standard_price
-				iline_id = self.pool.get('account.invoice.line').create(cr, uid, {
-					'name': p.name + ' - ' + line.name,
-					'invoice_id': invoice_id,
-					'uos_id': line.product_uos.id,
-					'product_id': line.product_id.id,
+				invoice_vals = {
+					'name': picking.name,
+					'origin': picking.name + ':' + picking.origin,
+					'type': type,
 					'account_id': account_id,
-					'price_unit': line.sale_line_id and line.sale_line_id.price_unit or line.product_id.list_price,
-					'quantity': line.product_uos_qty,
-					'invoice_line_tax_id': [(6,0,tax_ids)]
-				})
-				if line.sale_line_id:
-					self.pool.get('sale.order.line').write(cr, uid, [line.sale_line_id.id], {
-						'invoice_lines': [(6, 0, [iline_id])]
-					})
+					'partner_id': partner.id,
+					'address_invoice_id': address_invoice_id,
+					'address_contact_id': address_contact_id,
+					'comment': comment,
+					'payment_term': payment_term_id,
+					}
+				if journal_id:
+					invoice_vals['journal_id'] = journal_id
+				invoice_id = invoice_obj.create(cursor, user, invoice_vals,
+						context=context)
+				invoices_group[partner.id] = invoice_id
+			res[picking.id] = invoice_id
 
-			self.pool.get('stock.picking').write(cr, uid, [p.id], {'invoice_state': 'invoiced'})
-			if p.sale_id:
-				sids = sales[p.sale_id.id]
-				if invoice_id not in sids:
-					sales[p.sale_id.id].append(invoice_id)
-					self.pool.get('sale.order').write(cr, uid, [p.sale_id.id], {
-						'invoice_ids': [(6, 0, sales[p.sale_id.id])]
-					})
+			for move_line in picking.move_lines:
+				if group:
+					name = picking.name + '-' + move_line.name
+				else:
+					name = move_line.name
 
-				self.pool.get('sale.order').write(cr, uid, [p.sale_id.id], {
-					'invoiced': True
-				})
+				if type in ('out_invoice', 'out_refund'):
+					account_id = move_line.product_id.product_tmpl_id.\
+							property_account_income.id
+					if not account_id:
+						account_id = move_line.product_id.categ_id.\
+								property_account_income_categ.id
+				else:
+					account_id = move_line.product_id.product_tmpl_id.\
+							property_account_expense.id
+					if not account_id:
+						account_id = move_line.product_id.categ_id.\
+								property_account_expense_categ.id
+
+				price_unit = self._get_price_unit_invoice(cursor, user,
+						move_line, type)
+				discount = self._get_discount_invoice(cursor, user, move_line)
+				tax_ids = self._get_taxes_invoice(cursor, user, move_line, type)
+				account_analytic_id = self._get_account_analytic_invoice(cursor,
+						user, picking, move_line)
+
+				invoice_line_id = invoice_line_obj.create(cursor, user, {
+					'name': name,
+					'invoice_id': invoice_id,
+					'uos_id': move_line.product_uos.id,
+					'product_id': move_line.product_id.id,
+					'account_id': account_id,
+					'price_unit': price_unit,
+					'discount': discount,
+					'quantity': move_line.product_uos_qty,
+					'invoice_line_tax_id': [(6, 0, tax_ids)],
+					'account_analytic_id': account_analytic_id,
+					}, context=context)
+				self._invoice_line_hook(cursor, user, move_line, invoice_line_id)
+
+			invoice_obj.button_compute(cursor, user, [invoice_id], context=context,
+					set_total=(type in ('in_invoice', 'in_refund')))
+			self.write(cursor, user, [picking.id], {
+				'invoice_state': 'invoiced',
+				}, context=context)
+			self._invoice_hook(cursor, user, picking, invoice_id)
+		self.write(cursor, user, res.keys(), {
+			'invoice_state': 'invoiced',
+			}, context=context)
 		return res
+
 stock_picking()
 
+
 class stock_production_lot(osv.osv):
 
 	def name_get(self, cr, uid, ids, context={}):
@@ -574,7 +664,7 @@ class stock_production_lot(osv.osv):
 	_description = 'Production lot'
 
 	_columns = {
-		'name': fields.char('Code', size=64, required=True),
+		'name': fields.char('Serial', size=64, required=True),
 		'ref': fields.char('Reference', size=64),
 		'date': fields.datetime('Date create', required=True),
 		'revisions': fields.one2many('stock.production.lot.revision','lot_id','Revisions'),
@@ -583,6 +673,10 @@ class stock_production_lot(osv.osv):
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 		'name': lambda x,y,z,c: x.pool.get('ir.sequence').get(y,z,'stock.lot.serial'),
 	}
+	_sql_constraints = [
+		('name_ref_uniq', 'unique (name, ref)', 'The serial/ref must be unique !'),
+	]
+
 stock_production_lot()
 
 class stock_production_lot_revision(osv.osv):
@@ -624,32 +718,34 @@ class stock_move(osv.osv):
 		'priority': fields.selection([('0','Not urgent'),('1','Urgent')], 'Priority'),
 
 		'date': fields.datetime('Date Created'),
-		'date_planned': fields.date('Planned date', required=True),
+		'date_planned': fields.date('Scheduled date', required=True),
 
 		'product_id': fields.many2one('product.product', 'Product', required=True),
 
-		'product_qty': fields.float('Quantity (UOM)', required=True),
+		'product_qty': fields.float('Quantity', required=True),
 		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
 		'product_uos_qty': fields.float('Quantity (UOS)'),
 		'product_uos': fields.many2one('product.uom', 'Product UOS'),
-		'product_packaging' : fields.many2one('product.packaging', 'Product packaging'),
+		'product_packaging' : fields.many2one('product.packaging', 'Packaging'),
 
-		'location_id': fields.many2one('stock.location', 'Location', required=True),
+		'location_id': fields.many2one('stock.location', 'Source Location', required=True),
 		'location_dest_id': fields.many2one('stock.location', 'Dest. Location', required=True),
 		'address_id' : fields.many2one('res.partner.address', 'Dest. Address'),
 
-		'prodlot_id' : fields.many2one('stock.production.lot', 'Production lot'),
-		'tracking_id': fields.many2one('stock.tracking', 'Lot Tracking', select=True),
-		'lot_id': fields.many2one('stock.lot', 'Lot', select=True),
+		'prodlot_id' : fields.many2one('stock.production.lot', 'Production lot', help="Production lot is used to put a serial number on the production"),
+		'tracking_id': fields.many2one('stock.tracking', 'Tracking lot', select=True, help="Tracking lot is the code that will be put on the logistic unit/pallet"),
+		'lot_id': fields.many2one('stock.lot', 'Consumer lot', select=True, readonly=True),
 
 		'move_dest_id': fields.many2one('stock.move', 'Dest. Move'),
 		'move_history_ids': fields.many2many('stock.move', 'stock_move_history_ids', 'parent_id', 'child_id', 'Move History'),
 		'move_history_ids2': fields.many2many('stock.move', 'stock_move_history_ids', 'child_id', 'parent_id', 'Move History'),
-		'picking_id': fields.many2one('stock.picking', 'Picking list', select=True),
+		'picking_id': fields.many2one('stock.picking', 'Packing list', select=True),
 
 		'note': fields.text('Notes'),
 
 		'state': fields.selection([('draft','Draft'),('waiting','Waiting'),('confirmed','Confirmed'),('assigned','Assigned'),('done','Done'),('cancel','cancel')], 'State', readonly=True, select=True),
+		'price_unit': fields.float('Unit Price',
+			digits=(16, int(config['price_accuracy']))),
 	}
 	_defaults = {
 		'state': lambda *a: 'draft',
@@ -658,6 +754,18 @@ class stock_move(osv.osv):
 		'date_planned': lambda *a: time.strftime('%Y-%m-%d'),
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 	}
+
+	def _auto_init(self, cursor):
+		super(stock_move, self)._auto_init(cursor)
+		cursor.execute('SELECT indexname \
+				FROM pg_indexes \
+				WHERE indexname = \'stock_move_location_id_location_dest_id_product_id_state\'')
+		if not cursor.fetchone():
+			cursor.execute('CREATE INDEX stock_move_location_id_location_dest_id_product_id_state \
+					ON stock_move (location_id, location_dest_id, product_id, state)')
+			cursor.commit()
+
+
 	def onchange_product_id(self, cr, uid, context, prod_id=False, loc_id=False, loc_dest_id=False):
 		if not prod_id:
 			return {}
@@ -688,6 +796,10 @@ class stock_move(osv.osv):
 		self.write(cr, uid, ids, {'state' : 'assigned'})
 		return True
 
+	def cancel_assign(self, cr, uid, ids, context={}):
+		self.write(cr, uid, ids, {'state': 'confirmed'})
+		return True
+
 	#
 	# Duplicate stock.move
 	#
@@ -697,7 +809,8 @@ class stock_move(osv.osv):
 		pickings = {}
 		for move in self.browse(cr, uid, ids):
 			if move.product_id.type == 'consu':
-				done.append(move.id)
+				if mode.state in ('confirmed', 'waiting'):
+					done.append(move.id)
 				pickings[move.picking_id.id] = 1
 				continue
 			if move.state in ('confirmed','waiting'):
@@ -768,34 +881,86 @@ class stock_move(osv.osv):
 			#
 			# Accounting Entries
 			#
-#			acc_src = None
-#			acc_dest = None
-#			if move.location_id.account_id:
-#				acc_src =  move.location_id.account_id.id
-#			if move.location_dest_id.account_id:
-#				acc_dest =  move.location_dest_id.account_id.id
-#			if acc_src or acc_dest:
-#				test = [('product.product', move.product_id.id)]
-#				if move.product_id.categ_id:
-#					test.append( ('product.category', move.product_id.categ_id.id) )
-#				if not acc_src:
-#					acc_src = move.product_id.product_tmpl_id.property_account_expense[0]
-#				if not acc_dest:
-#					acc_dest = move.product_id.product_tmpl_id.property_account_income[0]
-#				if acc_src != acc_dest:
-#					amount = move.product_qty * move.product_id.standard_price
-#					lines = [
-#						(0,0,{'name': 'Stock', 'amount': -amount, 'account_id': acc_src}),
-#						(0,0,{'name': 'Stock', 'amount': amount, 'account_id': acc_dest})
-#					]
-#					self.pool.get('account.move').create(cr, uid, {'name': 'Stock', 'type':'undefined', 'line_id': lines})
+			acc_src = None
+			acc_dest = None
+			if move.location_id.account_id:
+				acc_src =  move.location_id.account_id.id
+			if move.location_dest_id.account_id:
+				acc_dest =  move.location_dest_id.account_id.id
+			if acc_src or acc_dest:
+				test = [('product.product', move.product_id.id)]
+				if move.product_id.categ_id:
+					test.append( ('product.category', move.product_id.categ_id.id) )
+				if not acc_src:
+					acc_src = move.product_id.product_tmpl_id.\
+							property_stock_account_input.id
+					if not acc_src:
+						acc_src = move.product_id.categ_id.\
+								property_stock_account_input_categ.id
+					if not acc_src:
+						raise osv.except_osv('Error!',
+								'There is no stock input account defined ' \
+										'for this product: "%s" (id: %d)' % \
+										(move.product_id.name,
+											move.product_id.id,))
+				if not acc_dest:
+					acc_dest = move.product_id.product_tmpl_id.\
+							property_stock_account_output.id
+					if not acc_dest:
+						acc_dest = move.product_id.categ_id.\
+								property_stock_account_output_categ.id
+					if not acc_dest:
+						raise osv.except_osv('Error!',
+								'There is no stock output account defined ' \
+										'for this product: "%s" (id: %d)' % \
+										(move.product_id.name,
+											move.product_id.id,))
+				if not move.product_id.categ_id.property_stock_journal.id:
+					raise osv.except_osv('Error!',
+							'There is no journal defined '\
+							'on the product category: "%s" (id: %d)' % \
+							(move.product_id.categ_id.name,
+								move.product_id.categ_id.id,))
+				journal_id = move.product_id.categ_id.property_stock_journal.id
+				if acc_src != acc_dest:
+					ref = move.picking_id and move.picking_id.name or False
+
+					if move.product_id.cost_method == 'average' and move.price_unit:
+						amount = move.product_qty * move.price_unit
+					else:
+						amount = move.product_qty * move.product_id.standard_price
+
+					date = time.strftime('%Y-%m-%d')
+					lines = [
+							(0, 0, {
+								'name': move.name,
+								'quantity': move.product_qty,
+								'credit': amount,
+								'account_id': acc_src,
+								'ref': ref,
+								'date': date}),
+							(0, 0, {
+								'name': move.name,
+								'quantity': move.product_qty,
+								'debit': amount,
+								'account_id': acc_dest,
+								'ref': ref,
+								'date': date})
+					]
+					self.pool.get('account.move').create(cr, uid,
+							{
+								'name': move.name,
+								'journal_id': journal_id,
+								'line_id': lines,
+								'ref': ref,
+							})
 		self.write(cr, uid, ids, {'state':'done'})
 
 		wf_service = netsvc.LocalService("workflow")
 		for id in ids:
 			wf_service.trg_trigger(uid, 'stock.move', id, cr)
-
 		return True
+
 stock_move()
 
 class stock_inventory(osv.osv):
@@ -826,7 +991,7 @@ class stock_inventory(osv.osv):
 				amount=self.pool.get('stock.location')._product_get(cr, uid, line.location_id.id, [pid], {'uom': line.product_uom.id})[pid]
 				change=line.product_qty-amount
 				if change:
-					location_id = line.product_id.product_tmpl_id.property_stock_inventory[0]
+					location_id = line.product_id.product_tmpl_id.property_stock_inventory.id
 					value = {
 						'name': 'INV:'+str(line.inventory_id.id)+':'+line.inventory_id.name,
 						'product_id': line.product_id.id,
@@ -906,6 +1071,15 @@ class product_product(osv.osv):
 	#
 	# Utiliser browse pour limiter les queries !
 	#
+	def view_header_get(self, cr, user, view_id, view_type, context):
+		if (not context.get('location', False)):
+			return False
+		cr.execute('select name from stock_location where id=%d', (context['location'],))
+		j = cr.fetchone()[0]
+		if j:
+			return 'Products: '+j
+		return False
+
 	def _get_product_available_func(states, what):
 		def _product_available(self, cr, uid, ids, name, arg, context={}):
 			if context.get('shop', False):
