@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: wizard_track_line.py 5828 2007-03-09 16:23:57Z ced $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -57,14 +57,17 @@ fields = {
 def _track_lines(self, cr, uid, data, context):
 	move_id = data['id']
 
-	prodlot_obj = pooler.get_pool(cr.dbname).get('stock.production.lot')
-	move_obj = pooler.get_pool(cr.dbname).get('stock.move')
-	sequence = pooler.get_pool(cr.dbname).get('ir.sequence').get(cr, uid, 'stock.lot.serial')
-	production_obj = pooler.get_pool(cr.dbname).get('mrp.production')
+	pool = pooler.get_pool(cr.dbname)
+	prodlot_obj = pool.get('stock.production.lot')
+	move_obj = pool.get('stock.move')
+	production_obj = pool.get('mrp.production')
+	ir_sequence_obj = pool.get('ir.sequence')
+
+	sequence = ir_sequence_obj.get(cr, uid, 'stock.lot.serial')
+	if not sequence:
+		raise wizard.except_wizard('Error!', 'No production sequence defined')
 	if data['form']['tracking_prefix']:
 		sequence=data['form']['tracking_prefix']+'/'+(sequence or '')
-	if not sequence:
-		raise wizard.except_wizard('Error !', 'No tracking prefix defined')
 
 	move = move_obj.browse(cr, uid, [move_id])[0]
 	quantity=data['form']['quantity']
@@ -86,7 +89,7 @@ def _track_lines(self, cr, uid, data, context):
 			new_move.append(current_move)
 		else:
 			current_move = move.id
-		new_prodlot = prodlot_obj.create(cr, uid, {'name': sequence, 'ref': '%d'%idx})
+		new_prodlot = prodlot_obj.create(cr, uid, {'name': sequence, 'ref': '%d'%idx}, {'product_id': move.product_id.id})
 		update_val['prodlot_id'] = new_prodlot
 		move_obj.write(cr, uid, [current_move], update_val)
 		production_ids = production_obj.search(cr, uid, [('move_lines', 'in', [move.id])])
