@@ -46,6 +46,8 @@ import netsvc
 import psycopg
 import warnings
 
+import tools
+
 def _symbol_set(symb):
 	if symb==None or symb==False:
 		return None
@@ -64,8 +66,8 @@ class _column(object):
 	_symbol_set = (_symbol_c, _symbol_f)
 	_symbol_get = None
 
-	def __init__(self, string='unknown', required=False, readonly=False, domain=[], context='', states={}, priority=0, change_default=False, size=None, ondelete="set null", translate=False, select=False, **args):
-		self.states = states
+	def __init__(self, string='unknown', required=False, readonly=False, domain=None, context='', states=None, priority=0, change_default=False, size=None, ondelete="set null", translate=False, select=False, **args):
+		self.states = states or {}
 		self.string = string
 		self.readonly = readonly
 		self.required = required
@@ -75,7 +77,7 @@ class _column(object):
 		self.change_default = change_default
 		self.ondelete = ondelete
 		self.translate = translate
-		self._domain = domain
+		self._domain = domain or []
 		self.relate =False
 		self._context = context
 		self.group_name = False
@@ -84,12 +86,17 @@ class _column(object):
 		for a in args:
 			if args[a]:
 				setattr(self, a, args[a])
+		if self.relate:
+			warnings.warn("The relate attribute doesn't work anymore, use act_window tag instead", DeprecationWarning)
 
-	def set(self, cr, obj, id, name, value, user=None, context={}):
+	def restart(self):
+		pass
+
+	def set(self, cr, obj, id, name, value, user=None, context=None):
 		cr.execute('update '+obj._table+' set '+name+'='+self._symbol_set[0]+' where id=%d', (self._symbol_set[1](value),id) )
 
-	def get(self, cr, obj, ids, name, context={}, values={}):
-		raise 'undefined get method !'
+	def get(self, cr, obj, ids, name, context=None, values=None):
+		raise Exception, 'undefined get method !'
 
 	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None):
 		ids = obj.search(cr, uid, args+self._domain+[(name,'ilike',value)], offset, limit)
@@ -142,7 +149,7 @@ class char(_column):
 			u_symb = unicode(symb)
 			
 		if len(u_symb) > self.size:
-			return u_symb[:self.size-3].encode('utf8') + '...'
+			raise Exception, '%s size greater than %d' % (self.string, self.size)
 		else:
 			return u_symb.encode('utf8')
 
@@ -207,12 +214,14 @@ class selection(_column):
 		_column.__init__(self, string=string, **args)
 		self.selection = selection
 
-	def set(self, cr, obj, id, name, value, user=None, context={}):
+	def set(self, cr, obj, id, name, value, user=None, context=None):
+		if not context:
+			context={}
 #CHECKME: a priori, ceci n'est jamais appel puisque le test ci-dessous est mauvais
 # la raison est que selection n'est pas en classic_write = false
 # a noter qu'on pourrait fournir un _symbol_set specifique, et ca suffirait
 		if value in self.selection:
-			raise 'BAD VALUE'
+			raise Exception, 'BAD VALUE'
 		_column.set(self, cr, obj, id, name, value, user=None, context=context)
 
 
@@ -239,7 +248,9 @@ class one2one(_column):
 		_column.__init__(self, string=string, **args)
 		self._obj = obj
 
-	def set(self, cr, obj_src, id, field, act, user=None, context={}):
+	def set(self, cr, obj_src, id, field, act, user=None, context=None):
+		if not context:
+			context={}
 		obj = obj_src.pool.get(self._obj)
 		self._table = obj_src.pool.get(self._obj)._table
 		if act[0]==0:
@@ -267,7 +278,11 @@ class many2one(_column):
 	# TODO: speed improvement
 	#
 	# name is the name of the relation field
-	def get(self, cr, obj, ids, name, user=None, context={}, values={}):
+	def get(self, cr, obj, ids, name, user=None, context=None, values=None):
+		if not context:
+			context={}
+		if not values:
+			values={}
 		res = {}
 		for r in values:
 			res[r['id']] = r[name]
@@ -275,7 +290,19 @@ class many2one(_column):
 			res.setdefault(id, '')
 		obj = obj.pool.get(self._obj)
 		# build a dictionary of the form {'id_of_distant_resource': name_of_distant_resource}
-		names = dict(obj.name_get(cr, user, filter(None, res.values()), context))
+		from orm import except_orm
+		try:
+			names = dict(obj.name_get(cr, user, filter(None, res.values()), context))
+		except except_orm:
+			names={}
+			for id in filter(None, res.values()):
+				try:
+					names[id] = dict(obj.name_get(cr, user, [id], context))[id]
+				except except_orm, e:
+					if e.name == 'AccessError':
+						names[id] = "== Access denied =="
+					else :
+						raise
 		for r in res.keys():
 			if res[r] and res[r] in names:
 				res[r] = (res[r], names[res[r]])
@@ -283,7 +310,9 @@ class many2one(_column):
 				res[r] = False
 		return res
 
-	def set(self, cr, obj_src, id, field, values, user=None, context={}):
+	def set(self, cr, obj_src, id, field, values, user=None, context=None):
+		if not context:
+			context={}
 		obj = obj_src.pool.get(self._obj)
 		self._table = obj_src.pool.get(self._obj)._table
 		if type(values)==type([]):
@@ -321,7 +350,11 @@ class one2many(_column):
 		#one2many can't be used as condition for defaults
 		assert(self.change_default != True)
 	
-	def get(self, cr, obj, ids, name, user=None, offset=0, context={}, values={}):
+	def get(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
+		if not context:
+			context = {}
+		if not values:
+			values = {}
 		res = {}
 		for id in ids:
 			res[id] = []
@@ -330,7 +363,9 @@ class one2many(_column):
 			res[r[self._fields_id]].append( r['id'] )
 		return res
 
-	def set(self, cr, obj, id, field, values, user=None, context={}):
+	def set(self, cr, obj, id, field, values, user=None, context=None):
+		if not context:
+			context={}
 		if not values:
 			return
 		_table = obj.pool.get(self._obj)._table
@@ -350,7 +385,12 @@ class one2many(_column):
 			elif act[0]==5:
 				cr.execute('update '+_table+' set '+self._fields_id+'=null where '+self._fields_id+'=%d', (id,))
 			elif act[0]==6:
-				if len(act[2]):
+				if not act[2]:
+					ids2 = [0]
+				else:
+					ids2 = act[2]
+				cr.execute('update '+_table+' set '+self._fields_id+'=NULL where '+self._fields_id+'=%d and id not in ('+','.join(map(str, ids2))+')', (id,))
+				if act[2]:
 					cr.execute('update '+_table+' set '+self._fields_id+'=%d where id in ('+','.join(map(str, act[2]))+')', (id,))
 	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None, operator='like'):
 		return obj.pool.get(self._obj).name_search(cr, uid, value, self._domain, offset, limit)
@@ -377,20 +417,37 @@ class many2many(_column):
 		self._id2 = id2
 		self._limit = limit
 
-	def get(self, cr, obj, ids, name, user=None, offset=0, context={}, values={}):
+	def get(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
+		if not context:
+			context={}
+		if not values:
+			values={}
 		res = {}
 		if not ids:
 			return res
-		ids_s = ','.join(map(str,ids))
-		limit_str = self._limit is not None and ' limit %d' % self._limit or ''
-		cr.execute('select '+self._id2+','+self._id1+' from '+self._rel+' where '+self._id1+' in ('+ids_s+')'+limit_str+' offset %d', (offset,))
 		for id in ids:
 			res[id] = []
+		ids_s = ','.join(map(str,ids))
+		limit_str = self._limit is not None and ' limit %d' % self._limit or ''
+		obj = obj.pool.get(self._obj)
+
+		d1, d2 = obj.pool.get('ir.rule').domain_get(cr, user, obj._name)
+		if d1:
+			d1 = ' and '+d1
+
+		cr.execute('SELECT '+self._rel+'.'+self._id2+','+self._rel+'.'+self._id1+' \
+				FROM '+self._rel+' , '+obj._table+' \
+				WHERE '+self._rel+'.'+self._id1+' in ('+ids_s+') \
+					AND '+self._rel+'.'+self._id2+' = '+obj._table+'.id '+d1
+				+limit_str+' order by '+obj._table+'.'+obj._order+' offset %d',
+				d2+[offset])
 		for r in cr.fetchall():
 			res[r[1]].append(r[0])
 		return res
 
-	def set(self, cr, obj, id, name, values, user=None, context={}):
+	def set(self, cr, obj, id, name, values, user=None, context=None):
+		if not context:
+			context={}
 		if not values:
 			return
 		obj = obj.pool.get(self._obj)
@@ -403,14 +460,19 @@ class many2many(_column):
 			elif act[0]==2:
 				obj.unlink(cr, user, [act[1]], context=context)
 			elif act[0]==3:
-				cr.execute('update '+self._rel+' set '+self._id2+'=null where ' +  self._id1 + '=%d', (act[1],))
+				cr.execute('delete from '+self._rel+' where ' +  self._id1 + '=%d and '+ self._id2 + '=%d', (id,act[1]))
 			elif act[0]==4:
-				cr.execute('update '+self._rel+' set '+self._id2+'=%d where ' + self._id1 + '=%d and ' + self._id2 + '=%d', (act[1], id, act[1]))
+				cr.execute('insert into '+self._rel+' ('+self._id1+','+self._id2+') values (%d,%d)', (id,act[1]))
 			elif act[0]==5:
 				cr.execute('update '+self._rel+' set '+self._id2+'=null where '+self._id2+'=%d', (id,))
 			elif act[0]==6:
-				cr.execute('delete from '+self._rel+' where '+self._id1+'=%d', (id, ))
-				for act_nbr in act[2]:
+
+				d1, d2 = obj.pool.get('ir.rule').domain_get(cr, user, obj._name) 
+				if d1:
+					d1 = ' and '+d1
+				cr.execute('delete from '+self._rel+' where '+self._id1+'=%d AND '+self._id2+' IN (SELECT '+self._rel+'.'+self._id2+' FROM '+self._rel+', '+obj._table+' WHERE '+self._rel+'.'+self._id1+'=%d AND '+self._rel+'.'+self._id2+' = '+obj._table+'.id '+ d1 +')', [id, id]+d2 )
+
+				for act_nbr in act[2]: 
 					cr.execute('insert into '+self._rel+' ('+self._id1+','+self._id2+') values (%d, %d)', (id, act_nbr))
 
 	#
@@ -429,7 +491,7 @@ class function(_column):
 	_type = 'function'
 	_properties = True
 	
-	def __init__(self, fnct, arg=None, fnct_inv=None, fnct_inv_arg=None, type='float', fnct_search=None, obj=None, method=False, **args):
+	def __init__(self, fnct, arg=None, fnct_inv=None, fnct_inv_arg=None, type='float', fnct_search=None, obj=None, method=False, store=False, **args):
 		_column.__init__(self, **args)
 		self._obj = obj
 		self._method = method
@@ -443,6 +505,11 @@ class function(_column):
 			self.readonly = 1
 		self._type = type
 		self._fnct_search = fnct_search
+		self.store = store
+		if type == 'float':
+			self._symbol_c = '%f'
+			self._symbol_f = lambda x: __builtin__.float(x or 0.0)
+			self._symbol_set = (self._symbol_c, self._symbol_f)
 		
 	def search(self, cr, uid, obj, name, args):
 		if not self._fnct_search:
@@ -450,7 +517,11 @@ class function(_column):
 			return []
 		return self._fnct_search(obj, cr, uid, obj, name, args)
 
-	def get(self, cr, obj, ids, name, user=None, context={}, values={}):
+	def get(self, cr, obj, ids, name, user=None, context=None, values=None):
+		if not context:
+			context={}
+		if not values:
+			values={}
 		res = {}
 		table = obj._table
 		if self._method:
@@ -459,7 +530,9 @@ class function(_column):
 		else:
 			return self._fnct(cr, table, ids, name, self._arg, context)
 
-	def set(self, cr, obj, id, name, value, user=None, context={}):
+	def set(self, cr, obj, id, name, value, user=None, context=None):
+		if not context:
+			context={}
 		if self._fnct_inv:
 			self._fnct_inv(obj, cr, user, id, name, value, self._fnct_inv_arg, context)
 
@@ -476,7 +549,9 @@ class serialized(_column):
 		super(serialized, self).__init__(string=string, **args)
 
 class property(function):
-	def _fnct_write(self2, self, cr, uid, id, prop, id_val, val, context={}):
+	def _fnct_write(self2, self, cr, uid, id, prop, id_val, val, context=None):
+		if not context:
+			context={}
 		(obj_dest,) = val
 		definition_id = self2._field_get(self, cr, uid, prop)
 
@@ -504,7 +579,9 @@ class property(function):
 			}, context=context)
 		return res
 
-	def _fnct_read(self2, self, cr, uid, ids, prop, val, context={}):
+	def _fnct_read(self2, self, cr, uid, ids, prop, val, context=None):
+		if not context:
+			context={}
 		property = self.pool.get('ir.property')
 		definition_id = self2._field_get(self, cr, uid, prop)
 
@@ -541,7 +618,7 @@ class property(function):
 
 	def __init__(self, obj_prop, **args):
 		self.field_id = {}
-		function.__init__(self,
-		  self._fnct_read, False,
-		  self._fnct_write, (obj_prop, ),
-		  **args)
+		function.__init__(self, self._fnct_read, False, self._fnct_write, (obj_prop, ), **args)
+
+	def restart(self):
+		self.field_id = {}
