@@ -54,16 +54,17 @@ def toxml(val):
 
 class report_int(netsvc.Service):
 	def __init__(self, name, audience='*'):
+		assert not netsvc.service_exist(name), 'The report "%s" already exist!'%name
 		super(report_int, self).__init__(name, audience)
 		if name[0:7]<>'report.':
-			raise 'ConceptionError, bad report name'
+			raise Exception, 'ConceptionError, bad report name, should start with "report."'
 		self.name = name
 		self.id = 0
 		self.name2 = '.'.join(name.split('.')[1:])
 		self.joinGroup('report')
 		self.exportMethod(self.create)
 
-	def create(self, cr, uid, ids, datas, context={}):
+	def create(self, cr, uid, ids, datas, context=None):
 		return False
 
 """
@@ -93,7 +94,9 @@ class report_rml(report_int):
 		pdf = create_doc(rml)
 		return (pdf, report_type)
 	
-	def create_xml(self, cr, uid, ids, datas, context={}):
+	def create_xml(self, cr, uid, ids, datas, context=None):
+		if not context:
+			context={}
 		doc = print_xml.document(cr, uid, datas, {})
 		self.bin_datas = doc.bin_datas
 		doc.parse(self.tmpl, ids, self.table, context)
@@ -101,7 +104,9 @@ class report_rml(report_int):
 		doc.close()
 		return self.post_process_xml_data(cr, uid, xml, context)
 
-	def post_process_xml_data(self, cr, uid, xml, context={}):
+	def post_process_xml_data(self, cr, uid, xml, context=None):
+		if not context:
+			context={}
 		# find the position of the 3rd tag 
 		# (skip the <?xml ...?> and the "root" tag)
 		iter = re.finditer('<[^>]*>', xml)
@@ -125,7 +130,9 @@ class report_rml(report_int):
 	#
 	# TODO: The translation doesn't work for "<tag t="1">textext<tag> tex</tag>text</tag>"
 	#
-	def create_rml(self, cr, xml, uid, context={}):
+	def create_rml(self, cr, xml, uid, context=None):
+		if not context:
+			context={}
 		service = netsvc.LocalService("object_proxy")
 
 		# In some case we might not use xsl ...
@@ -133,10 +140,19 @@ class report_rml(report_int):
 			return xml
 
 		# load XSL (parse it to the XML level)
-		styledoc = libxml2.parseFile(os.path.join(tools.config['root_path'],self.xsl))
-		
-		#TODO: get all the translation in one query. That means we have to: 
-		# * build a list of items to translate, 
+		styledoc = libxml2.parseDoc(tools.file_open(
+			os.path.join(tools.config['root_path'], self.xsl)).read())
+		xsl_path, tail = os.path.split(os.path.join(tools.config['root_path'],
+			self.xsl))
+		for child in styledoc.children:
+			if child.name == 'import':
+				if child.hasProp('href'):
+					file = child.prop('href')
+					child.setProp('href', str(
+						os.path.normpath(os.path.join(xsl_path, file))))
+
+		#TODO: get all the translation in one query. That means we have to:
+		# * build a list of items to translate,
 		# * issue the query to translate them,
 		# * (re)build/update the stylesheet with the translated items
 
@@ -145,7 +161,8 @@ class report_rml(report_int):
 			while child is not None:
 				if (child.type == "element") and child.hasProp('t'):
 					#FIXME: use cursor
-					res = service.execute(cr.dbname, uid, 'ir.translation', '_get_source', self.name2, 'xsl', lang, child.content)
+					res = service.execute(cr.dbname, uid, 'ir.translation',
+							'_get_source', self.name2, 'xsl', lang, child.content)
 					if res:
 						child.setContent(res)
 				look_down(child.children, lang)
@@ -154,11 +171,14 @@ class report_rml(report_int):
 		if context.get('lang', False):
 			look_down(styledoc.children, context['lang'])
 
-		style = libxslt.parseStylesheetDoc(styledoc)			# parse XSL
-
-		doc = libxml2.parseMemory(xml,len(xml))					# load XML (data)
-		result = style.applyStylesheet(doc, None)				# create RML (apply XSL to XML data)
-		xml = style.saveResultToString(result)					# save result to string
+		# parse XSL
+		style = libxslt.parseStylesheetDoc(styledoc)
+		# load XML (data)
+		doc = libxml2.parseMemory(xml,len(xml))
+		# create RML (apply XSL to XML data)
+		result = style.applyStylesheet(doc, None)
+		# save result to string
+		xml = style.saveResultToString(result)
 		
 		style.freeStylesheet()
 		doc.freeDoc()
@@ -188,8 +208,10 @@ def register_all(db):
 	result = cr.dictfetchall()
 	cr.close()
 	for r in result:
+		if netsvc.service_exist('report.'+r['report_name']):
+			continue
 		if r['report_rml']:
-			report_sxw('report.'+r['report_name'], r['model'], opj('addons',r['report_rml']))
+			report_sxw('report.'+r['report_name'], r['model'], opj('addons',r['report_rml']), header=r['header'])
 		if r['report_xsl']:
 			report_rml('report.'+r['report_name'], r['model'], opj('addons',r['report_xml']), r['report_xsl'] and opj('addons',r['report_xsl']))
 
