@@ -57,22 +57,32 @@ class report_printscreen_list(report_int):
 		return result
 
 	def _parse_string(self, view):
-		dom = minidom.parseString(view)
+		dom = minidom.parseString(view.encode('utf-8'))
 		return self._parse_node(dom)
 
-	def create(self, cr, uid, ids, datas, context={}):
+	def create(self, cr, uid, ids, datas, context=None):
+		if not context:
+			context={}
+		pool = pooler.get_pool(cr.dbname)
+		model = pool.get(datas['model'])
+		model_id = pool.get('ir.model').search(cr, uid, [('model','=',model._name)])
+		if model_id:
+			model_desc = pool.get('ir.model').browse(cr, uid, model_id[0], context).name
+		else:
+			model_desc = model._description
+
 		datas['ids'] = ids
 		model = pooler.get_pool(cr.dbname).get(datas['model'])
-		result = model.fields_view_get(cr, uid, view_type='tree', context=context)
 
+		result = model.fields_view_get(cr, uid, view_type='tree', context=context)
 		fields_order = self._parse_string(result['arch'])
-		rows = model.read(cr, uid, datas['ids'], result['fields'].keys() )
-		res = self._create_table(uid, datas['ids'], result['fields'], fields_order, rows, context, model._description)
+		rows = model.read(cr, uid, datas['ids'], result['fields'].keys(), context )
+		res = self._create_table(uid, datas['ids'], result['fields'], fields_order, rows, context, model_desc)
 		return (self.obj.get(), 'pdf')
 
 
 	def _create_table(self, uid, ids, fields, fields_order, results, context, title=''):
-		pageSize=[297.0,210.0]
+		pageSize=[297.0, 210.0]
 
 		impl = minidom.getDOMImplementation()
 		new_doc = impl.createDocument(None, "report", None)
@@ -92,59 +102,101 @@ class report_printscreen_list(report_int):
 		_append_node('PageHeight', '%.2f' %(pageSize[1] * 2.8346,))
 
 		_append_node('report-header', title)
-
 		l = []
 		t = 0
+		rowcount=0;
 		strmax = (pageSize[0]-40) * 2.8346
+		temp = []
+		count = len(fields_order)
+		for i in range(0,count):
+			temp.append(0)
+
+		ince = -1;
 		for f in fields_order:
 			s = 0
+			ince += 1
 			if fields[f]['type'] in ('date','time','float','integer'):
 				s = 60
 				strmax -= s
+				if fields[f]['type'] in ('float','integer'):
+					temp[ince]=1;
 			else:
-				t += fields[f].get('size', 56) / 28 + 1
+				t += fields[f].get('size', 80) / 28 + 1
+
 			l.append(s)
+
 		for pos in range(len(l)):
 			if not l[pos]:
-				s = fields[fields_order[pos]].get('size', 56) / 28 + 1
+				s = fields[fields_order[pos]].get('size', 80) / 28 + 1
 				l[pos] = strmax * s / t
+
 		_append_node('tableSize', ','.join(map(str,l)) )
 		new_doc.childNodes[0].appendChild(config)
 		header = new_doc.createElement("header")
 
 		for f in fields_order:
 			field = new_doc.createElement("field")
-			field_txt = new_doc.createTextNode('%s' % (fields[f]['string'],))
+			field_txt = new_doc.createTextNode(str(fields[f]['string'] or ''))
 			field.appendChild(field_txt)
 			header.appendChild(field)
-		
+
 		new_doc.childNodes[0].appendChild(header)
 
 		lines = new_doc.createElement("lines")
+
+		tsum = []
+		count = len(fields_order)
+		for i in range(0,count):
+			tsum.append(0)
+
 		for line in results:
 			node_line = new_doc.createElement("row")
+
+			count = -1
 			for f in fields_order:
+				count += 1
 				if fields[f]['type']=='many2one' and line[f]:
 					line[f] = line[f][1]
 				if fields[f]['type'] in ('one2many','many2many') and line[f]:
 					line[f] = '( '+str(len(line[f])) + ' )'
 				col = new_doc.createElement("col")
+				col.setAttribute('para','yes')
 				col.setAttribute('tree','no')
 				if line[f] != None:
 					txt = new_doc.createTextNode(str(line[f] or ''))
+					if temp[count] == 1:
+						tsum[count] = tsum[count] + line[f];
+
 				else:
 					txt = new_doc.createTextNode('/')
 				col.appendChild(txt)
 				node_line.appendChild(col)
 			lines.appendChild(node_line)
+		node_line = new_doc.createElement("row")
+		lines.appendChild(node_line)
+		node_line = new_doc.createElement("row")
+		for f in range(0,count):
+			col = new_doc.createElement("col")
+			col.setAttribute('para','yes')
+			col.setAttribute('tree','no')
+			if tsum[f] != None:
+				txt = new_doc.createTextNode(str(tsum[f] or ''))
+			else:
+				txt = new_doc.createTextNode('/')
+			if f == 0:
+				txt = new_doc.createTextNode('Total')
+
+			col.appendChild(txt)
+			node_line.appendChild(col)
+		lines.appendChild(node_line)
+
 		new_doc.childNodes[0].appendChild(lines)
 
 		styledoc = libxml2.parseFile(os.path.join(tools.config['root_path'],'addons/base/report/custom_new.xsl'))
 		style = libxslt.parseStylesheetDoc(styledoc)
 		doc = libxml2.parseDoc(new_doc.toxml())
 		rml_obj = style.applyStylesheet(doc, None)
-		rml = style.saveResultToString(rml_obj) 
-
+		rml = style.saveResultToString(rml_obj)
 		self.obj = render.rml(rml)
 		self.obj.render()
 		return True
