@@ -37,7 +37,7 @@ import os
 # Change this to UTF-8 if you plan tu use Reportlab's UTF-8 support
 #
 # reportlab use "code page 1252" encoding by default. cfr reportlab user guide p.46
-encoding = 'cp1252'
+encoding = 'utf-8'
 
 def str2xml(s):
 	return s.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
@@ -160,6 +160,20 @@ class _rml_doc(object):
 				addMapping(name, 1, 0, name)    #bold
 				addMapping(name, 1, 1, name)    #italic and bold
 
+	def _textual_image(self, node):
+		import base64
+		rc = ''
+		for n in node.childNodes:
+			if n.nodeType in (node.CDATA_SECTION_NODE, node.TEXT_NODE):
+				rc += n.data
+		return base64.decodestring(rc)
+
+	def _images(self, el):
+		result = {}
+		for node in el.getElementsByTagName('image'):
+			result[node.getAttribute('name')] = self._textual_image(node)
+		return result
+
 	def render(self, out):
 		el = self.dom.documentElement.getElementsByTagName('docinit')
 		if el:
@@ -168,6 +182,10 @@ class _rml_doc(object):
 		el = self.dom.documentElement.getElementsByTagName('stylesheet')
 		self.styles = _rml_styles(el)
 
+		el = self.dom.documentElement.getElementsByTagName('images')
+		if el:
+			self.images.update( self._images(el[0]) )
+
 		el = self.dom.documentElement.getElementsByTagName('template')
 		if len(el):
 			pt_obj = _rml_template(out, el[0], self, images=self.images, path=self.path)
@@ -285,12 +303,15 @@ class _rml_canvas(object):
 		if not node.hasAttribute('file'):
 			s = self.images[node.getAttribute('name')]
 		else:
-			try:
-				u = urllib.urlopen(str(node.getAttribute('file')))
-				s = StringIO.StringIO(u.read())
-			except:
-				u = file(os.path.join(self.path,str(node.getAttribute('file'))), 'rb')
-				s = StringIO.StringIO(u.read())
+			if node.getAttribute('file') in self.images:
+				s = StringIO.StringIO(self.images[node.getAttribute('file')])
+			else:
+				try:
+					u = urllib.urlopen(str(node.getAttribute('file')))
+					s = StringIO.StringIO(u.read())
+				except:
+					u = file(os.path.join(self.path,str(node.getAttribute('file'))), 'rb')
+					s = StringIO.StringIO(u.read())
 		img = ImageReader(s)
 		(sx,sy) = img.getSize()
 
@@ -517,9 +538,13 @@ class _rml_flowable(object):
 			return platypus.Paragraph(self._textual(node), style, **(utils.attr_get(node, [], {'bulletText':'str'})))
 		elif node.localName=='image':
 			if not node.hasAttribute('file'):
-				name = '/tmp/image_%d.jpg' % (int(node.getAttribute('name')),)
-				file(name,'wb+').write( self.doc.images[node.getAttribute('name')].read() )
-				return platypus.Image(name, mask=(250,255,250,255,250,255), **(utils.attr_get(node, ['width','height'])))
+				if node.hasAttribute('name'):
+					image_data = self.doc.images[node.getAttribute('name')].read()
+				else:
+					import base64
+					image_data = base64.decodestring(node.firstChild.nodeValue)
+				image = StringIO.StringIO(image_data)
+				return platypus.Image(image, mask=(250,255,250,255,250,255), **(utils.attr_get(node, ['width','height'])))
 			else:
 				return platypus.Image(node.getAttribute('file'), mask=(250,255,250,255,250,255), **(utils.attr_get(node, ['width','height'])))
 
@@ -559,6 +584,13 @@ class _rml_flowable(object):
 			return platypus.NextPageTemplate(str(node.getAttribute('name')))
 		elif node.localName=='nextFrame':
 			return platypus.CondPageBreak(1000)           # TODO: change the 1000 !
+		elif node.localName == 'currentFrame':
+			#return platypus.CondPageBreak(1000)           # TODO: change the 1000 !
+			from reportlab.platypus.doctemplate import CurrentFrameFlowable
+			return CurrentFrameFlowable(str(node.getAttribute('name')))
+		elif node.localName == 'frameEnd':
+			return False
+			return EndFrameFlowable()
 		else:
 			sys.stderr.write('Warning: flowable not yet implemented: %s !\n' % (node.localName,))
 			return None
@@ -577,6 +609,31 @@ class _rml_flowable(object):
 			node = node.nextSibling
 		return story
 
+from reportlab.platypus.doctemplate import ActionFlowable
+
+class EndFrameFlowable(ActionFlowable):
+	def __init__(self,resume=0):
+		ActionFlowable.__init__(self,('frameEnd',resume))
+
+class TinyDocTemplate(platypus.BaseDocTemplate):
+
+	def ___handle_pageBegin(self):
+		self.page = self.page + 1
+		self.pageTemplate.beforeDrawPage(self.canv,self)
+		self.pageTemplate.checkPageSize(self.canv,self)
+		self.pageTemplate.onPage(self.canv,self)
+		for f in self.pageTemplate.frames: f._reset()
+		self.beforePage()
+		#keep a count of flowables added to this page.  zero indicates bad stuff
+		self._curPageFlowableCount = 0
+		if hasattr(self,'_nextFrameIndex'):
+			del self._nextFrameIndex
+		for f in self.pageTemplate.frames:
+			if f.id == 'first':
+				self.frame = f
+				break
+		self.handle_frameBegin()
+
 class _rml_template(object):
 	def __init__(self, out, node, doc, images={}, path='.'):
 		self.images= images
@@ -587,7 +644,7 @@ class _rml_template(object):
 			ps = map(lambda x:x.strip(), node.getAttribute('pageSize').replace(')', '').replace('(', '').split(','))
 			pageSize = ( utils.unit_get(ps[0]),utils.unit_get(ps[1]) )
 		cm = reportlab.lib.units.cm
-		self.doc_tmpl = platypus.BaseDocTemplate(out, pagesize=pageSize, **utils.attr_get(node, ['leftMargin','rightMargin','topMargin','bottomMargin'], {'allowSplitting':'int','showBoundary':'bool','title':'str','author':'str'}))
+		self.doc_tmpl = TinyDocTemplate(out, pagesize=pageSize, **utils.attr_get(node, ['leftMargin','rightMargin','topMargin','bottomMargin'], {'allowSplitting':'int','showBoundary':'bool','title':'str','author':'str'}))
 		self.page_templates = []
 		self.styles = doc.styles
 		self.doc = doc
@@ -595,7 +652,7 @@ class _rml_template(object):
 		for pt in pts:
 			frames = []
 			for frame_el in pt.getElementsByTagName('frame'):
-				frame = platypus.Frame( **(utils.attr_get(frame_el, ['x1','y1', 'width','height', 'leftPadding', 'rightPadding', 'bottomPadding', 'topPadding'], {'id':'text', 'showBoundary':'bool'})) )
+				frame = platypus.Frame( **(utils.attr_get(frame_el, ['x1','y1', 'width','height', 'leftPadding', 'rightPadding', 'bottomPadding', 'topPadding'], {'id':'str', 'showBoundary':'bool'})) )
 				frames.append( frame )
 			gr = pt.getElementsByTagName('pageGraphics')
 			if len(gr):
