@@ -34,22 +34,22 @@ _uid_cache = {}
 
 def login(db, login, password):
 	cr = pooler.get_db(db).cursor()
-	cr.execute('select id from res_users where login=%s and password=%s and active', (login, password))
+	cr.execute('select id from res_users where login=%s and password=%s and active', (login.encode('utf-8'), password.encode('utf-8')))
 	res = cr.fetchone()
 	cr.close()
 	if res:
 		return res[0]
 	else:
-		raise _('Bad username or password')
+		return False
 
 def check_super(passwd):
 	if passwd == tools.config['admin_passwd']:
 		return True
 	else:
-		raise 'AccessDenied'
+		raise Exception('AccessDenied')
 	
 def check(db, uid, passwd):
-	#FIXME: this should be db dependent
+	# XXX FIXME: this should be db dependent
 	if _uid_cache.has_key(uid) and (_uid_cache[uid]==passwd):
 		return True
 	cr = pooler.get_db(db).cursor()
@@ -57,7 +57,7 @@ def check(db, uid, passwd):
 	res = cr.fetchone()[0]
 	cr.close()
 	if not bool(res):
-		raise 'AccessDenied'
+		raise Exception('AccessDenied')
 	if res:
 		_uid_cache[uid] = passwd
 	return bool(res)
@@ -68,5 +68,5 @@ def access(db, uid, passwd, sec_level, ids):
 	res = cr.fetchone()
 	cr.close()
 	if not res:
-		raise _('Bad username or password')
+		raise Exception('Bad username or password')
 	return res[0]
