@@ -38,9 +38,9 @@ GNU Public Licence.
 
 (c) 2003-TODAY, Fabien Pinckaers - Tiny sprl
 """
-
-__author__ = 'Fabien Pinckaers, <fp@tiny.be>'
-__version__ = '4.0.3'
+import release
+__author__ = release.author
+__version__ = release.version
 
 import __builtin__
 __builtin__.__dict__['tinyerp_version'] = __version__
@@ -51,7 +51,7 @@ __builtin__.__dict__['tinyerp_version_string'] = "Tiny ERP Server " + __version_
 #----------------------------------------------------------
 # python imports
 #----------------------------------------------------------
-import os, signal, sys
+import sys,os,signal
 
 #----------------------------------------------------------
 # get logger
@@ -66,10 +66,10 @@ logger = netsvc.Logger()
 # import the tools module so that the commandline parameters are parsed
 #-----------------------------------------------------------------------
 import tools
+import time
 
 if sys.platform=='win32':
 	import mx.DateTime
-	import time
 	mx.DateTime.strptime = lambda x,y: mx.DateTime.mktime(time.strptime(x, y))
 
 #os.chdir(tools.file_path_root)
@@ -179,7 +179,7 @@ if tools.config["translate_out"]:
 
 	logger.notifyChannel("init", netsvc.LOG_INFO, 'writing translation file for language %s to %s' % (tools.config["language"], tools.config["translate_out"]))
 	trans=tools.trans_generate(tools.config["language"], tools.config["translate_modules"])
-	writer=csv.writer(file(tools.config["translate_out"], "w"))
+	writer=csv.writer(file(tools.config["translate_out"], "w"), 'TINY')
 	for row in trans:
 		writer.writerow(row)
 	del trans
@@ -196,37 +196,76 @@ if tools.config["translate_in"]:
 if tools.config["stop_after_init"]:
     sys.exit(0)
 
+
 #----------------------------------------------------------
 # Launch Server
 #----------------------------------------------------------
-try:
-	port = int(tools.config["port"])
-except Exception:
-	logger.notifyChannel("init", netsvc.LOG_ERROR, "invalid port '%s'!" % (tools.config["port"],))
-	sys.exit(1)
-interface = tools.config["interface"]
-secure = tools.config["secure"]
 
-httpd = netsvc.HttpDaemon(interface,port, secure)
+if tools.config['xmlrpc']:
+	try:
+		port = int(tools.config["port"])
+	except Exception:
+		logger.notifyChannel("init", netsvc.LOG_ERROR, "invalid port '%s'!" % (tools.config["port"],))
+		sys.exit(1)
+	interface = tools.config["interface"]
+	secure = tools.config["secure"]
+	
+	httpd = netsvc.HttpDaemon(interface,port, secure)
+	
+	if tools.config["xmlrpc"]:
+		xml_gw = netsvc.xmlrpc.RpcGateway('web-services')
+		httpd.attach("/xmlrpc", xml_gw )
+		logger.notifyChannel("web-services", netsvc.LOG_INFO, "starting XML-RPC services, port "+str(port))
+	
+	#
+	#if tools.config["soap"]:
+	#	soap_gw = netsvc.xmlrpc.RpcGateway('web-services')
+	#	httpd.attach("/soap", soap_gw )
+	#	logger.notifyChannel("web-services", netsvc.LOG_INFO, 'starting SOAP services, port '+str(port))
+	#
+	if not (netsvc.HAS_SSL and tools.config['secure']):
+		logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are not using the SSL layer")
+	else:
+		logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are using the SSL Layer")
 
-if tools.config["xmlrpc"]:
-	xml_gw = netsvc.xmlrpc.RpcGateway('web-services')
-	httpd.attach("/xmlrpc", xml_gw )
-	logger.notifyChannel("web-services", netsvc.LOG_INFO, "starting XML-RPC services, port "+str(port))
+if tools.config['netrpc']:
+	try:
+		netport = int(tools.config["netport"])
+	except Exception:
+		logger.notifyChannel("init", netsvc.LOG_ERROR, "invalid port '%s'!" % (tools.config["netport"],))
+		sys.exit(1)
+	netinterface = tools.config["netinterface"]
+	
+	tinySocket = netsvc.TinySocketServerThread(netinterface, netport, False)
+	logger.notifyChannel("web-services", netsvc.LOG_INFO, "starting netrpc service, port "+str(netport))
 
-#
-#if tools.config["soap"]:
-#	soap_gw = netsvc.xmlrpc.RpcGateway('web-services')
-#	httpd.attach("/soap", soap_gw )
-#	logger.notifyChannel("web-services", netsvc.LOG_INFO, 'starting SOAP services, port '+str(port))
-#
-if not (netsvc.HAS_SSL and tools.config['secure']):
-	logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are not using the SSL layer")
-else:
-	logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are using the SSL Layer")
+def handler(signum, frame):
+	from tools import config
+	if tools.config['netrpc']:
+		tinySocket.stop()
+	if tools.config['xmlrpc']:
+		httpd.stop()
+	netsvc.Agent.quit()
+	if config['pidfile']:
+		os.unlink(config['pidfile'])
+	sys.exit(0)
+
+from tools import config
+if config['pidfile']:
+	fd=open(config['pidfile'], 'w')
+	pidtext="%d" % (os.getpid())
+	fd.write(pidtext)
+	fd.close()
+
+signal.signal(signal.SIGINT, handler)
+signal.signal(signal.SIGTERM, handler)
 
 logger.notifyChannel("web-services", netsvc.LOG_INFO, 'the server is running, waiting for connections...')
+if tools.config['netrpc']:
+	tinySocket.start()
+if tools.config['xmlrpc']:
+	httpd.start()
+#dispatcher.run()
 
-
-httpd.start()
-dispatcher.run()
+while True:
+	time.sleep(1)
